from unittest.mock import patch

from pyfakefs.fake_pathlib import FakePathlibModule  # type: ignore
import pytest

from clinvar_this import config, exceptions

CONFIG_CONTENT = """[default]
auth_token = "MYTOKEN"
verify_ssl = true
"""


def test_config():
    short_config = config.Config(profile="default", auth_token="123")
    assert str(short_config) == "Config(profile='default', auth_token='***', verify_ssl=True)"
    long_config = config.Config(profile="default", auth_token="1234567890")
    assert str(long_config) == "Config(profile='default', auth_token='12345*****', verify_ssl=True)"


def test_load_config_success(fs):
    fake_pathlib = FakePathlibModule(fs)

    with patch("clinvar_this.config.pathlib", fake_pathlib):
        base_path = config.pathlib.Path.home() / ".config" / "clinvar-this"
        base_path.mkdir(parents=True)
        fs.create_file(
            (base_path / "config.toml"), contents=CONFIG_CONTENT, create_missing_dirs=True
        )
        config_obj = config.load_config(profile="default")

    assert str(config_obj) == "Config(profile='default', auth_token='MYTOK**', verify_ssl=True)"


def test_load_config_fail_missing_config(fs):
    with pytest.raises(exceptions.ConfigFileMissingException):
        config.load_config(profile="default")


def test_load_config_fail_invalid_toml(fs):
    fake_pathlib = FakePathlibModule(fs)

    with patch("clinvar_this.config.pathlib", fake_pathlib):
        base_path = config.pathlib.Path.home() / ".config" / "clinvar-this"
        fs.create_file(
            (base_path / "config.toml"),
            contents=CONFIG_CONTENT + "x/asdf",
            create_missing_dirs=True,
        )
        with pytest.raises(exceptions.ConfigException) as e:
            config.load_config(profile="doesnotexist")
        assert "'Problem decoding configuration file" in str(e)


def test_save_config_fresh(fs):
    fake_pathlib = FakePathlibModule(fs)

    with patch("clinvar_this.config.pathlib", fake_pathlib):
        base_path = config.pathlib.Path.home() / ".config" / "clinvar-this"
        config.save_config(config=config.Config("default", "xxx"))

        with (base_path / "config.toml").open("rt") as inputf:
            config_str = inputf.read()

    assert config_str == CONFIG_CONTENT.replace("MYTOKEN", "xxx")


def test_save_config_overwrite(fs):
    fake_pathlib = FakePathlibModule(fs)

    with patch("clinvar_this.config.pathlib", fake_pathlib):
        base_path = config.pathlib.Path.home() / ".config" / "clinvar-this"
        base_path.mkdir(parents=True)
        fs.create_file(
            (base_path / "config.toml"), contents=CONFIG_CONTENT, create_missing_dirs=True
        )
        config.save_config(config=config.Config("default", "xxx"))

        with (base_path / "config.toml").open("rt") as inputf:
            config_str = inputf.read()

    assert config_str == CONFIG_CONTENT.replace("MYTOKEN", "xxx")


def test_dump_config_success(fs_config, capsys):
    config.dump_config()

    captured = capsys.readouterr()
    assert "fake" in captured.out


def test_dump_config_no_file(fs, capsys):
    config.dump_config()

    captured = capsys.readouterr()
    assert "# no file at" in captured.out



"""Test ``clinvar_api.models`` with unit tests."""

import datetime

from clinvar_api import models, msg


def test_created_construction():
    models.Created(id="test-id")


def test_created_from_msg():
    models.Created.from_msg(msg.Created(id="test-id"))


def test_error_construction():
    models.Error(message="fake-message")


def test_error_from_msg():
    models.Error.from_msg(msg.Error(message="fake-message"))


def test_submission_status_file_construction():
    models.SubmissionStatusFile(url="http://example.com")


def test_submission_status_file_from_msg():
    models.SubmissionStatusFile.from_msg(msg.SubmissionStatusFile(url="http://example.com"))


def test_submission_status_object_content_construction():
    models.SubmissionStatusObjectContent(
        clinvar_processing_status="In processing",
        clinvar_release_status="Not released",
    )


def test_submission_status_object_content_from_msg():
    models.SubmissionStatusObjectContent.from_msg(
        msg.SubmissionStatusObjectContent(
            clinvarProcessingStatus="In processing",
            clinvarReleaseStatus="Not released",
        )
    )


def test_submission_status_object_construction():
    models.SubmissionStatusObject(
        accession="accession",
        content=models.SubmissionStatusObjectContent(
            clinvar_processing_status="In processing",
            clinvar_release_status="Not released",
        ),
        target_db="clinvar",
    )
    models.SubmissionStatusObject(
        accession=None,
        content=models.SubmissionStatusObjectContent(
            clinvar_processing_status="In processing",
            clinvar_release_status="Not released",
        ),
        target_db="clinvar",
    )


def test_submission_status_object_from_msg():
    models.SubmissionStatusObject.from_msg(
        msg.SubmissionStatusObject(
            accession="accession",
            content=msg.SubmissionStatusObjectContent(
                clinvarProcessingStatus="In processing",
                clinvarReleaseStatus="Not released",
            ),
            targetDb="clinvar",
        )
    )


def test_submission_status_response_message_construction():
    models.SubmissionStatusResponseMessage(
        error_code="error-code", severity="fake-severity", text="fake text"
    )
    models.SubmissionStatusResponseMessage(
        error_code=None, severity="fake-severity", text="fake text"
    )


def test_submission_status_response_message_from_msg():
    models.SubmissionStatusResponseMessage.from_msg(
        msg.SubmissionStatusResponseMessage(
            errorCode="error-code", severity="fake-severity", text="fake text"
        )
    )


def test_submission_status_response_construction():
    models.SubmissionStatusResponse(
        status=None,
        files=[models.SubmissionStatusFile(url="http://example.com")],
        message=models.SubmissionStatusResponseMessage(
            error_code="error-code", severity="fake-severity", text="fake text"
        ),
        objects=[
            models.SubmissionStatusObject(
                accession=None,
                content=models.SubmissionStatusObjectContent(
                    clinvar_processing_status="In processing",
                    clinvar_release_status="Not released",
                ),
                target_db="clinvar",
            )
        ],
    )
    models.SubmissionStatusResponse(
        status="fake-status",
        files=[models.SubmissionStatusFile(url="http://example.com")],
        message=models.SubmissionStatusResponseMessage(
            error_code="error-code", severity="fake-severity", text="fake text"
        ),
        objects=[
            models.SubmissionStatusObject(
                accession=None,
                content=models.SubmissionStatusObjectContent(
                    clinvar_processing_status="In processing",
                    clinvar_release_status="Not released",
                ),
                target_db="clinvar",
            )
        ],
    )


def test_submission_status_response_from_msg():
    models.SubmissionStatusResponse.from_msg(
        msg.SubmissionStatusResponse(
            status=None,
            files=[msg.SubmissionStatusFile(url="http://example.com")],
            message=msg.SubmissionStatusResponseMessage(
                errorCode="error-code", severity="fake-severity", text="fake text"
            ),
            objects=[
                msg.SubmissionStatusObject(
                    accession=None,
                    content=msg.SubmissionStatusObjectContent(
                        clinvarProcessingStatus="In processing",
                        clinvarReleaseStatus="Not released",
                    ),
                    targetDb="clinvar",
                )
            ],
        )
    )


def test_submission_status_actions_construction():
    models.SubmissionStatusActions(
        id="fake-id",
        responses=[
            models.SubmissionStatusResponse(
                status=None,
                files=[models.SubmissionStatusFile(url="http://example.com")],
                message=models.SubmissionStatusResponseMessage(
                    error_code="error-code", severity="fake-severity", text="fake text"
                ),
                objects=[
                    models.SubmissionStatusObject(
                        accession=None,
                        content=models.SubmissionStatusObjectContent(
                            clinvar_processing_status="In processing",
                            clinvar_release_status="Not released",
                        ),
                        target_db="clinvar",
                    )
                ],
            )
        ],
        status="the-status",
        target_db="clinvar",
        updated=datetime.datetime.now(),
    )
    models.SubmissionStatusActions(
        id="fake-id",
        responses=[],
        status="the-status",
        target_db="clinvar",
        updated=datetime.datetime.now(),
    )


def test_submission_status_actions_from_msg():
    models.SubmissionStatusActions.from_msg(
        msg.SubmissionStatusActions(
            id="fake-id",
            responses=[
                msg.SubmissionStatusResponse(
                    status=None,
                    files=[msg.SubmissionStatusFile(url="http://example.com")],
                    message=msg.SubmissionStatusResponseMessage(
                        errorCode="error-code", severity="fake-severity", text="fake text"
                    ),
                    objects=[
                        msg.SubmissionStatusObject(
                            accession=None,
                            content=msg.SubmissionStatusObjectContent(
                                clinvarProcessingStatus="In processing",
                                clinvarReleaseStatus="Not released",
                            ),
                            targetDb="clinvar",
                        )
                    ],
                )
            ],
            status="the-status",
            targetDb="clinvar",
            updated=datetime.datetime.now(),
        )
    )


def test_submission_status_construction():
    models.SubmissionStatus(actions=[])
    models.SubmissionStatus(
        actions=[
            models.SubmissionStatusActions(
                id="fake-id",
                responses=[],
                status="the-status",
                target_db="clinvar",
                updated=datetime.datetime.now(),
            )
        ]
    )


def test_submission_status_from_msg():
    models.SubmissionStatus.from_msg(
        msg.SubmissionStatus(
            actions=[
                msg.SubmissionStatusActions(
                    id="fake-id",
                    responses=[],
                    status="the-status",
                    targetDb="clinvar",
                    updated=datetime.datetime.now(),
                )
            ]
        )
    )


def test_summary_response_error_input_construction():
    models.SummaryResponseErrorInput(value="value", field="field")
    models.SummaryResponseErrorInput(value="value", field=None)


def test_summary_response_error_input_from_msg():
    models.SummaryResponseErrorInput.from_msg(
        msg.SummaryResponseErrorInput(value="value", field="field")
    )


def test_summary_response_error_output_error_construction():
    models.SummaryResponseErrorOutputError(user_message="the message")


def test_summary_response_error_output_error_from_msg():
    models.SummaryResponseErrorOutputError.from_msg(
        msg.SummaryResponseErrorOutputError(userMessage="the message")
    )


def test_summary_response_error_output_construction():
    models.SummaryResponseErrorOutput(
        errors=[models.SummaryResponseErrorOutputError(user_message="the message")]
    )
    models.SummaryResponseErrorOutput(errors=[])


def test_summary_response_error_output_from_msg():
    models.SummaryResponseErrorOutput.from_msg(
        msg.SummaryResponseErrorOutput(
            errors=[msg.SummaryResponseErrorOutputError(userMessage="the message")]
        )
    )


def test_summary_response_error_construction():
    models.SummaryResponseError(
        input=[models.SummaryResponseErrorInput(value="value", field="field")],
        output=models.SummaryResponseErrorOutput(errors=[]),
    )
    models.SummaryResponseError(input=[], output=models.SummaryResponseErrorOutput(errors=[]))


def test_summary_response_error_from_msg():
    models.SummaryResponseError.from_msg(
        msg.SummaryResponseError(
            input=[msg.SummaryResponseErrorInput(value="value", field="field")],
            output=msg.SummaryResponseErrorOutput(errors=[]),
        )
    )


def test_summary_response_deletion_identifier_construction():
    models.SummaryResponseDeletionIdentifier(
        clinvar_accession="the-accession",
        clinvar_local_key="the-local-key",
    )
    models.SummaryResponseDeletionIdentifier(
        clinvar_accession="the-accession",
        clinvar_local_key=None,
    )


def test_summary_response_deletion_identifier_from_msg():
    models.SummaryResponseDeletionIdentifier.from_msg(
        msg.SummaryResponseDeletionIdentifier(
            clinvarAccession="the-accession",
            clinvarLocalKey="the-local-key",
        )
    )


def test_summary_response_deletion_construction():
    models.SummaryResponseDeletion(
        identifiers=models.SummaryResponseDeletionIdentifier(
            clinvar_accession="the-accession",
        ),
        processing_status="processing-status",
        delete_date="2022-01-01",
        delete_status="delete-status",
        errors=[
            models.SummaryResponseError(
                input=[], output=models.SummaryResponseErrorOutput(errors=[])
            )
        ],
    )
    models.SummaryResponseDeletion(
        identifiers=models.SummaryResponseDeletionIdentifier(
            clinvar_accession="the-accession",
        ),
        processing_status="processing-status",
        delete_date=None,
        delete_status=None,
        errors=None,
    )


def test_summary_response_deletion_from_msg():
    models.SummaryResponseDeletion.from_msg(
        msg.SummaryResponseDeletion(
            identifiers=msg.SummaryResponseDeletionIdentifier(
                clinvarAccession="the-accession",
            ),
            processingStatus="processing-status",
            deleteDate="2022-01-01",
            deleteStatus="delete-status",
            errors=[
                msg.SummaryResponseError(input=[], output=msg.SummaryResponseErrorOutput(errors=[]))
            ],
        )
    )


def test_summary_response_submission_identifier_construction():
    models.SummaryResponseSubmissionIdentifiers(
        clinvar_local_key="local-key",
        clinvar_accession="the-accession",
        local_id="local-id",
        local_key="local-key",
    )
    models.SummaryResponseSubmissionIdentifiers(
        clinvar_local_key="local-key",
        clinvar_accession=None,
        local_id=None,
        local_key=None,
    )


def test_summary_response_submission_identifier_from_msg():
    models.SummaryResponseSubmissionIdentifiers.from_msg(
        msg.SummaryResponseSubmissionIdentifiers(
            clinvarLocalKey="local-key",
            clinvarAccession="the-accession",
            localID="local-id",
            localKey="local-key",
        )
    )


def test_summary_response_submission_construction():
    models.SummaryResponseSubmission(
        identifiers=models.SummaryResponseSubmissionIdentifiers(clinvar_local_key="local-key"),
        processing_status="processing-status",
        clinvar_accession_version="accession-version",
        errors=[
            models.SummaryResponseError(
                input=[], output=models.SummaryResponseErrorOutput(errors=[])
            )
        ],
        release_date="2022-01-01",
        release_status="released",
    )
    models.SummaryResponseSubmission(
        identifiers=models.SummaryResponseSubmissionIdentifiers(clinvar_local_key="local-key"),
        processing_status="processing-status",
        clinvar_accession_version=None,
        errors=None,
        release_date=None,
        release_status=None,
    )


def test_summary_response_submission_from_msg():
    models.SummaryResponseSubmission.from_msg(
        msg.SummaryResponseSubmission(
            identifiers=msg.SummaryResponseSubmissionIdentifiers(clinvarLocalKey="local-key"),
            processingStatus="processing-status",
            clinvarAccessionVersion="accession-version",
            errors=[
                msg.SummaryResponseError(input=[], output=msg.SummaryResponseErrorOutput(errors=[]))
            ],
            releaseDate="2022-01-01",
            releaseStatus="released",
        )
    )


def test_summary_response_construction():
    models.SummaryResponse(
        batch_processing_status="status",
        batch_release_status="released",
        submission_date="2022-01-01",
        submission_name="submission-name",
        total_count=10,
        total_errors=10,
        total_public=10,
        total_success=10,
        deletions=[
            models.SummaryResponseDeletion(
                identifiers=models.SummaryResponseDeletionIdentifier(
                    clinvar_accession="the-accession",
                ),
                processing_status="processing-status",
            )
        ],
        submissions=[
            models.SummaryResponseSubmission(
                identifiers=models.SummaryResponseSubmissionIdentifiers(
                    clinvar_local_key="local-key"
                ),
                processing_status="processing-status",
            )
        ],
        total_delete_count=10,
        total_deleted=10,
        total_delete_errors=10,
        total_delete_success=10,
    )
    models.SummaryResponse(
        batch_processing_status="status",
        batch_release_status="released",
        submission_date="2022-01-01",
        submission_name="submission-name",
        total_count=10,
        total_errors=10,
        total_public=10,
        total_success=10,
        deletions=None,
        submissions=None,
        total_delete_count=None,
        total_deleted=None,
        total_delete_errors=None,
        total_delete_success=None,
    )


def test_summary_response_from_msg():
    models.SummaryResponse.from_msg(
        msg.SummaryResponse(
            batchProcessingStatus="status",
            batchReleaseStatus="released",
            submissionDate="2022-01-01",
            submissionName="submission-name",
            totalCount=10,
            totalErrors=10,
            totalPublic=10,
            totalSuccess=10,
            deletions=[
                msg.SummaryResponseDeletion(
                    identifiers=msg.SummaryResponseDeletionIdentifier(
                        clinvarAccession="the-accession",
                    ),
                    processingStatus="processing-status",
                )
            ],
            submissions=[
                msg.SummaryResponseSubmission(
                    identifiers=msg.SummaryResponseSubmissionIdentifiers(
                        clinvarLocalKey="local-key"
                    ),
                    processingStatus="processing-status",
                )
            ],
            totalDeleteCount=10,
            totalDeleted=10,
            totalDeleteErrors=10,
            totalDeleteSuccess=10,
        )
    )


def test_sumbmission_clinvar_deletion_accession_set_construction():
    models.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason="whatever",
    )
    models.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason=None,
    )


def test_sumbmission_clinvar_deletion_accession_set_to_msg():
    models.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason="whatever",
    ).to_msg()
    models.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason=None,
    ).to_msg()


def test_submission_clinvar_deletion_construction():
    models.SubmissionClinvarDeletion(
        accession_set=[
            models.SubmissionClinvarDeletionAccessionSet(
                accession="accession",
                reason="whatever",
            )
        ]
    )
    models.SubmissionClinvarDeletion(accession_set=[])


def test_submission_clinvar_deletion_to_msg():
    models.SubmissionClinvarDeletion(
        accession_set=[
            models.SubmissionClinvarDeletionAccessionSet(
                accession="accession",
                reason="whatever",
            )
        ]
    ).to_msg()
    models.SubmissionClinvarDeletion(accession_set=[]).to_msg()


def test_submission_chromosome_coordinates_construction():
    models.SubmissionChromosomeCoordinates(
        accession="accession",
        alternate_allele="A",
        assembly=models.Assembly.GRCH37,
        chromosome=models.Chromosome.CHR12,
        inner_start=100,
        inner_stop=200,
        outer_start=90,
        outer_stop=210,
        reference_allele="C",
        start=100,
        stop=200,
        variant_length=10,
    )
    models.SubmissionChromosomeCoordinates(
        assembly=models.Assembly.GRCH37,
        chromosome=models.Chromosome.CHR12,
        start=100,
        stop=100,
        reference_allele="C",
        alternate_allele="A",
        variant_length=1,
    )


def test_submission_chromosome_coordinates_to_msg():
    models.SubmissionChromosomeCoordinates(
        accession="accession",
        alternate_allele="A",
        assembly=models.Assembly.GRCH37,
        chromosome=models.Chromosome.CHR12,
        inner_start=100,
        inner_stop=200,
        outer_start=90,
        outer_stop=210,
        reference_allele="C",
        start=100,
        stop=200,
        variant_length=10,
    ).to_msg()
    models.SubmissionChromosomeCoordinates(
        assembly=models.Assembly.GRCH37,
        chromosome=models.Chromosome.CHR12,
        start=100,
        stop=100,
        reference_allele="C",
        alternate_allele="A",
        variant_length=1,
    ).to_msg()


def test_submission_variant_gene_construction():
    models.SubmissionVariantGene(
        id="some-id",
        symbol="some-symbol",
    )
    models.SubmissionVariantGene()


def test_submission_variant_gene_to_msg():
    models.SubmissionVariantGene(
        id="some-id",
        symbol="some-symbol",
    ).to_msg()
    models.SubmissionVariantGene().to_msg()


def test_submission_variant_construction():
    models.SubmissionVariant(
        chromosome_coordinates=models.SubmissionChromosomeCoordinates(
            assembly=models.Assembly.GRCH37,
            chromosome=models.Chromosome.CHR12,
            start=100,
            stop=100,
            reference_allele="C",
            alternate_allele="A",
            variant_length=1,
        ),
        copy_number="2",
        gene=[models.SubmissionVariantGene()],
        hgvs="hgvs-string",
        reference_copy_number="2",
        variant_type=models.VariantType.DELETION,
    )
    models.SubmissionVariant()


def test_submission_variant_to_msg():
    models.SubmissionVariant(
        chromosome_coordinates=models.SubmissionChromosomeCoordinates(
            assembly=models.Assembly.GRCH37,
            chromosome=models.Chromosome.CHR12,
            start=100,
            stop=100,
            reference_allele="C",
            alternate_allele="A",
            variant_length=1,
        ),
        copy_number="2",
        gene=[models.SubmissionVariantGene()],
        hgvs="hgvs-string",
        reference_copy_number="2",
        variant_type=models.VariantType.DELETION,
    ).to_msg()
    models.SubmissionVariant().to_msg()


def test_sumbmission_variant_set_construction():
    models.SubmissionVariantSet(variant=[models.SubmissionVariant()])
    models.SubmissionVariantSet(variant=[])


def test_sumbmission_variant_set_to_msg():
    models.SubmissionVariantSet(variant=[models.SubmissionVariant()]).to_msg()
    models.SubmissionVariantSet(variant=[]).to_msg()


def test_submission_phase_unknown_set_construction():
    models.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[models.SubmissionVariant()],
    )
    models.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[],
    )


def test_submission_phase_unknown_set_to_msg():
    models.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[models.SubmissionVariant()],
    ).to_msg()
    models.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[],
    ).to_msg()


def test_submission_clinical_feature_construction():
    models.SubmissionClinicalFeature(
        clinical_features_affected_status="affected-status",
        db="HP",
        id="the-id",
        name="same-name",
    )


def test_submission_clinical_feature_to_msg():
    models.SubmissionClinicalFeature(
        clinical_features_affected_status="affected-status",
        db="HP",
        id="the-id",
        name="same-name",
    ).to_msg()


def test_submission_observed_in_construction():
    models.SubmissionObservedIn(
        affected_status="affected",
        allele_origin="germline",
        collection_method="clinical testing",
        clinical_features=[
            models.SubmissionClinicalFeature(
                clinical_features_affected_status="affected-status",
                db="HP",
                id="the-id",
                name="same-name",
            )
        ],
        clinical_features_comment="some comment",
        number_of_individuals=1,
        struct_var_method_type="some-type",
    )
    models.SubmissionObservedIn(
        affected_status="affected",
        allele_origin="germline",
        collection_method="clinical testing",
    )


def test_submission_observed_in_to_msg():
    models.SubmissionObservedIn(
        affected_status="affected",
        allele_origin="germline",
        collection_method="clinical testing",
        clinical_features=[
            models.SubmissionClinicalFeature(
                clinical_features_affected_status="affected-status",
                db="HP",
                id="the-id",
                name="same-name",
            )
        ],
        clinical_features_comment="some comment",
        number_of_individuals=1,
        struct_var_method_type="some-type",
    ).to_msg()
    models.SubmissionObservedIn(
        affected_status="affected",
        allele_origin="germline",
        collection_method="clinical testing",
    ).to_msg()


def test_submission_haplotype_set_construction():
    models.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[],
        star_allele_name="the-name",
    )
    models.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[models.SubmissionVariant()],
        star_allele_name="the-name",
    )


def test_submission_haplotype_set_to_msg():
    models.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[],
        star_allele_name="the-name",
    ).to_msg()
    models.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[models.SubmissionVariant()],
        star_allele_name="the-name",
    ).to_msg()


def test_submission_distinct_chromosome_set_construction():
    models.SubmissionDistinctChromosomesSet(
        hgvs="some-hgvs",
        variants=[
            models.SubmissionVariant(),
            models.SubmissionVariant(),
        ],
    )


def test_submission_distinct_chromosome_set_to_msg():
    models.SubmissionDistinctChromosomesSet(
        hgvs="some-hgvs",
        variants=[
            models.SubmissionVariant(),
            models.SubmissionVariant(),
        ],
    ).to_msg()


def test_submission_haplotype_sets_construction():
    models.SubmissionHaplotypeSets(
        haplotype_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
        haplotype_single_variant_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
    )
    models.SubmissionHaplotypeSets()


def test_submission_haplotype_sets_to_msg():
    models.SubmissionHaplotypeSets(
        haplotype_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
        haplotype_single_variant_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
    ).to_msg()
    models.SubmissionHaplotypeSets().to_msg()


def test_submission_diplotype_set_construction():
    models.SubmissionDiplotypeSet(
        haplotype_sets=[models.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        star_allele_name="star-allele-name",
    )
    models.SubmissionDiplotypeSet(
        haplotype_sets=[models.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        star_allele_name=None,
    )


def test_submission_diplotype_set_to_msg():
    models.SubmissionDiplotypeSet(
        haplotype_sets=[models.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        star_allele_name="star-allele-name",
    ).to_msg()
    models.SubmissionDiplotypeSet(
        haplotype_sets=[models.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        star_allele_name=None,
    ).to_msg()


def test_submission_citation_construction():
    models.SubmissionCitation(
        db=models.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    )
    models.SubmissionCitation(
        db=None,
        id=None,
        url=None,
    )


def test_submission_citation_to_msg():
    models.SubmissionCitation(
        db=models.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    ).to_msg()
    models.SubmissionCitation(
        db=None,
        id=None,
        url=None,
    ).to_msg()


def test_submission_assertion_criteria_construction():
    models.SubmissionAssertionCriteria(
        db=models.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    ).to_msg()
    models.SubmissionAssertionCriteria(
        db=None,
        id=None,
        url=None,
    ).to_msg()


def test_submission_assertion_criteria_to_msg():
    models.SubmissionAssertionCriteria(
        db=models.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    ).to_msg()


def test_submission_condition_construction():
    models.SubmissionCondition(
        db=models.ConditionDb.HP,
        id="some-id",
        name="some-name",
    )
    models.SubmissionCondition(
        db=None,
        id=None,
        name=None,
    )


def test_submission_condition_to_msg():
    models.SubmissionCondition(
        db=models.ConditionDb.HP,
        id="some-id",
        name="some-name",
    ).to_msg()
    models.SubmissionCondition(
        db=None,
        id=None,
        name=None,
    ).to_msg()


def test_submission_drug_response_construction():
    models.SubmissionDrugResponse(
        db=models.ConditionDb.HP,
        drug_name="some drug",
        id="some-id",
        condition=[models.SubmissionCondition()],
    )
    models.SubmissionDrugResponse(
        db=None,
        drug_name=None,
        id=None,
        condition=None,
    )


def test_submission_drug_response_to_msg():
    models.SubmissionDrugResponse(
        db=models.ConditionDb.HP,
        drug_name="some drug",
        id="some-id",
        condition=[models.SubmissionCondition()],
    ).to_msg()
    models.SubmissionDrugResponse(
        db=None,
        drug_name=None,
        id=None,
        condition=None,
    ).to_msg()


def test_submission_condition_set_construction():
    models.SubmissionConditionSet(
        condition=[models.SubmissionCondition()],
        drug_response=[models.SubmissionDrugResponse()],
    )
    models.SubmissionConditionSet(
        condition=None,
        drug_response=None,
    )


def test_submission_condition_set_to_msg():
    models.SubmissionConditionSet(
        condition=[models.SubmissionCondition()],
        drug_response=[models.SubmissionDrugResponse()],
    ).to_msg()
    models.SubmissionConditionSet(
        condition=None,
        drug_response=None,
    ).to_msg()


def test_submission_compound_heterozygote_set_variant_set_construction():
    models.SubmissionCompoundHeterozygoteSetVariantSet(
        variant_set=models.SubmissionVariantSet(variant=[]),
    )
    models.SubmissionCompoundHeterozygoteSetVariantSet(
        variant_set=None,
    )


def test_submission_compound_heterozygote_set_variant_set_to_msg():
    models.SubmissionCompoundHeterozygoteSetVariantSet(
        variant_set=models.SubmissionVariantSet(variant=[]),
    ).to_msg()
    models.SubmissionCompoundHeterozygoteSetVariantSet(
        variant_set=None,
    ).to_msg()


def test_submission_compound_heterozygote_set_construction():
    models.SubmissionCompoundHeterozygoteSet(
        hgvs="hgvs",
        variant_sets=[
            models.SubmissionCompoundHeterozygoteSetVariantSet(),
            models.SubmissionCompoundHeterozygoteSetVariantSet(),
        ],
    )


def test_submission_compound_heterozygote_set_to_msg():
    models.SubmissionCompoundHeterozygoteSet(
        hgvs="hgvs",
        variant_sets=[
            models.SubmissionCompoundHeterozygoteSetVariantSet(),
            models.SubmissionCompoundHeterozygoteSetVariantSet(),
        ],
    ).to_msg()


def test_submission_clinical_significance_construction():
    models.SubmissionClinicalSignificance(
        clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        citation=[
            models.SubmissionCitation(
                db=models.CitationDb.PMC,
                id="PMC123",
                url="https://example.com",
            )
        ],
        comment="some comment",
        custom_assertion_score=42.0,
        date_last_evaluated="2022-01-01",
        explanation_of_drug_response="some explanation",
        explanation_of_other_clinical_significance="2022-01-01",
        mode_of_inheritance=models.ModeOfInheritance.AUTOSOMAL_DOMINANT_INHERITANCE,
    )
    models.SubmissionClinicalSignificance(
        clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
    )


def test_submission_clinical_significance_to_msg():
    models.SubmissionClinicalSignificance(
        clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        citation=[
            models.SubmissionCitation(
                db=models.CitationDb.PMC,
                id="PMC123",
                url="https://example.com",
            )
        ],
        comment="some comment",
        custom_assertion_score=42.0,
        date_last_evaluated="2022-01-01",
        explanation_of_drug_response="some explanation",
        explanation_of_other_clinical_significance="2022-01-01",
        mode_of_inheritance=models.ModeOfInheritance.AUTOSOMAL_DOMINANT_INHERITANCE,
    ).to_msg()
    models.SubmissionClinicalSignificance(
        clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
    ).to_msg()


def test_submission_clinvar_submission_construction():
    models.SubmissionClinvarSubmission(
        clinical_significance=models.SubmissionClinicalSignificance(
            clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        condition_set=models.SubmissionConditionSet(),
        observed_in=[
            models.SubmissionObservedIn(
                affected_status="affected",
                allele_origin="germline",
                collection_method="clinical testing",
            )
        ],
        record_status="record-status",
        clinvar_accession="some-accession",
        compound_heterozygote_set=models.SubmissionCompoundHeterozygoteSet(
            hgvs="hgvs",
            variant_sets=[
                models.SubmissionCompoundHeterozygoteSetVariantSet(),
                models.SubmissionCompoundHeterozygoteSetVariantSet(),
            ],
        ),
        diplotype_set=models.SubmissionDiplotypeSet(
            haplotype_sets=[models.SubmissionHaplotypeSets()],
            hgvs="the-hgvs",
        ),
        distinct_chromosomes_set=models.SubmissionDistinctChromosomesSet(
            hgvs="some-hgvs",
            variants=[
                models.SubmissionVariant(),
                models.SubmissionVariant(),
            ],
        ),
        haplotype_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[],
            star_allele_name="the-name",
        ),
        haplotype_single_variant_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
        local_id="local-id",
        local_key="local-key",
        phase_unknown_set=models.SubmissionPhaseUnknownSet(
            hgvs="hgvs-string",
            variants=[],
        ),
        variant_set=models.SubmissionVariantSet(variant=[]),
    )
    models.SubmissionClinvarSubmission(
        clinical_significance=models.SubmissionClinicalSignificance(
            clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        condition_set=models.SubmissionConditionSet(),
        observed_in=[
            models.SubmissionObservedIn(
                affected_status="affected",
                allele_origin="germline",
                collection_method="clinical testing",
            )
        ],
        record_status="record-status",
    )


def test_submission_clinvar_submission_to_msg():
    models.SubmissionClinvarSubmission(
        clinical_significance=models.SubmissionClinicalSignificance(
            clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        condition_set=models.SubmissionConditionSet(),
        observed_in=[
            models.SubmissionObservedIn(
                affected_status="affected",
                allele_origin="germline",
                collection_method="clinical testing",
            )
        ],
        record_status="record-status",
        clinvar_accession="some-accession",
        compound_heterozygote_set=models.SubmissionCompoundHeterozygoteSet(
            hgvs="hgvs",
            variant_sets=[
                models.SubmissionCompoundHeterozygoteSetVariantSet(),
                models.SubmissionCompoundHeterozygoteSetVariantSet(),
            ],
        ),
        diplotype_set=models.SubmissionDiplotypeSet(
            haplotype_sets=[models.SubmissionHaplotypeSets()],
            hgvs="the-hgvs",
        ),
        distinct_chromosomes_set=models.SubmissionDistinctChromosomesSet(
            hgvs="some-hgvs",
            variants=[
                models.SubmissionVariant(),
                models.SubmissionVariant(),
            ],
        ),
        haplotype_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[],
            star_allele_name="the-name",
        ),
        haplotype_single_variant_set=models.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[models.SubmissionVariant()],
            star_allele_name="the-name",
        ),
        local_id="local-id",
        local_key="local-key",
        phase_unknown_set=models.SubmissionPhaseUnknownSet(
            hgvs="hgvs-string",
            variants=[],
        ),
        variant_set=models.SubmissionVariantSet(variant=[]),
    ).to_msg()
    models.SubmissionClinvarSubmission(
        clinical_significance=models.SubmissionClinicalSignificance(
            clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        condition_set=models.SubmissionConditionSet(),
        observed_in=[
            models.SubmissionObservedIn(
                affected_status="affected",
                allele_origin="germline",
                collection_method="clinical testing",
            )
        ],
        record_status="record-status",
    ).to_msg()


def test_submission_container_construction():
    models.SubmissionContainer(
        behalf_org_id=123,
        clinvar_deletion=models.SubmissionClinvarDeletion(
            accession_set=[
                models.SubmissionClinvarDeletionAccessionSet(
                    accession="accession",
                    reason="whatever",
                )
            ]
        ),
        clinvar_submission=None,
        clinvar_submission_release_status="release-status",
        submission_name="some-name",
    )
    models.SubmissionContainer(
        behalf_org_id=123,
        clinvar_deletion=None,
        clinvar_submission=[
            models.SubmissionClinvarSubmission(
                clinical_significance=models.SubmissionClinicalSignificance(
                    clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
                ),
                condition_set=models.SubmissionConditionSet(),
                observed_in=[
                    models.SubmissionObservedIn(
                        affected_status="affected",
                        allele_origin="germline",
                        collection_method="clinical testing",
                    )
                ],
                record_status="record-status",
            )
        ],
        clinvar_submission_release_status="release-status",
        submission_name="some-name",
    )


def test_submission_container_to_msg():
    models.SubmissionContainer(
        behalf_org_id=123,
        clinvar_deletion=models.SubmissionClinvarDeletion(
            accession_set=[
                models.SubmissionClinvarDeletionAccessionSet(
                    accession="accession",
                    reason="whatever",
                )
            ]
        ),
        clinvar_submission=None,
        clinvar_submission_release_status="release-status",
        submission_name="some-name",
    ).to_msg()
    models.SubmissionContainer(
        behalf_org_id=123,
        clinvar_deletion=None,
        clinvar_submission=[
            models.SubmissionClinvarSubmission(
                clinical_significance=models.SubmissionClinicalSignificance(
                    clinical_significance_description=models.ClinicalSignificanceDescription.PATHOGENIC,
                ),
                condition_set=models.SubmissionConditionSet(),
                observed_in=[
                    models.SubmissionObservedIn(
                        affected_status="affected",
                        allele_origin="germline",
                        collection_method="clinical testing",
                    )
                ],
                record_status="record-status",
            )
        ],
        clinvar_submission_release_status="release-status",
        submission_name="some-name",
    ).to_msg()



import pytest


@pytest.fixture
def data_created():
    return {"id": "SUB999999"}


@pytest.fixture
def data_message():
    return {"message": "No valid API key provided"}


@pytest.fixture
def data_submission_submitted():
    """Example data 'Example with a status of "submitted"'."""
    return {
        "actions": [
            {
                "id": "SUB999999-1",
                "responses": [],
                "status": "submitted",
                "targetDb": "clinvar",
                "updated": "2021-03-19T17:24:24.384085Z",
            }
        ]
    }


@pytest.fixture
def data_submission_processing():
    """Example data 'Example with a status of "processing"'."""
    return {
        "actions": [
            {
                "id": "SUB999999-1",
                "responses": [
                    {
                        "files": [],
                        "message": None,
                        "objects": [
                            {
                                "accession": None,
                                "content": {
                                    "clinvarProcessingStatus": "In processing",
                                    "clinvarReleaseStatus": "Not released",
                                },
                                "targetDb": "clinvar",
                            }
                        ],
                        "status": "processing",
                    }
                ],
                "status": "processing",
                "targetDb": "clinvar",
                "updated": "2021-03-19T12:33:09.243072Z",
            }
        ]
    }


@pytest.fixture
def data_submission_processed():
    """Example data 'Example of a successful submission with a status of "processed"'."""
    return {
        "actions": [
            {
                "id": "SUB999999-1",
                "responses": [
                    {
                        "status": "processed",
                        "message": {
                            "errorCode": None,
                            "severity": "info",
                            "text": (
                                'Your ClinVar submission processing status is "Success". Please find the '
                                "details in the file referenced by actions[0].responses[0].files[0].url."
                            ),
                        },
                        "files": [
                            {
                                "url": (
                                    "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx"
                                    "/sub999999-summary-report.json/?format=attachment"
                                )
                            }
                        ],
                        "objects": [],
                    }
                ],
                "status": "processed",
                "targetDb": "clinvar",
                "updated": "2021-03-24T04:22:04.101297Z",
            }
        ]
    }


@pytest.fixture
def data_partially_successful_submission():
    """Example data'Example of a partially successful submission and a status of "error"'."""
    return {
        "actions": [
            {
                "id": "SUB999999-1",
                "targetDb": "clinvar",
                "status": "error",
                "updated": "2021-03-25T16:05:03.319474Z",
                "responses": [
                    {
                        "status": "error",
                        "message": {
                            "severity": "error",
                            "errorCode": "1",
                            "text": (
                                'Your ClinVar submission processing status is "Partial success". '
                                "Please find the details in the file referenced by "
                                "actions[0].responses[0].files[0].url.",
                            ),
                        },
                        "files": [
                            {
                                "url": (
                                    "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx/"
                                    "sub999999-summary-report.json/?format=attachment"
                                )
                            }
                        ],
                        "objects": [],
                    }
                ],
            }
        ]
    }


@pytest.fixture
def data_submission_error():
    """Example data 'Example of a submission with a status of "error" where all records failed:'."""
    return {
        "actions": [
            {
                "id": "SUB999999-1",
                "targetDb": "clinvar",
                "status": "error",
                "updated": "2021-03-25T15:31:04.411550Z",
                "responses": [
                    {
                        "status": "error",
                        "message": {
                            "severity": "error",
                            "errorCode": "2",
                            "text": (
                                'Your ClinVar submission processing status is "Error".  Please find the '
                                "details in the file referenced by actions[0].responses[0].files[0].url."
                            ),
                        },
                        "files": [
                            {
                                "url": (
                                    "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx/"
                                    "sub999999-summary-report.json/?format=attachment"
                                )
                            }
                        ],
                        "objects": [],
                    }
                ],
            }
        ]
    }


@pytest.fixture
def data_summary_response_processed():
    """Example of a successful submission with a status of processed:

    This response contains a URL for a summary file with more details. Following is an example:
    """
    return {
        "submissionName": "SUB673156",
        "submissionDate": "2021-03-25",
        "batchProcessingStatus": "Success",
        "batchReleaseStatus": "Not released",
        "totalCount": 3,
        "totalErrors": 0,
        "totalSuccess": 3,
        "totalPublic": 0,
        "submissions": [
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "clinvarAccession": "SCV000839746",
                },
                "processingStatus": "Success",
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "clinvarAccession": "SCV000839747",
                },
                "processingStatus": "Success",
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3",
                    "clinvarAccession": "SCV000839748",
                },
                "processingStatus": "Success",
            },
        ],
    }


@pytest.fixture
def data_summary_response_error_partial():
    """Example of a partially successful submission and a status of error:

    This response contains a URL for a summary file with more details. Following is an example:
    """
    return {
        "submissionName": "PAHVCEP_10_2020_API",
        "submissionDate": "2021-03-25",
        "batchProcessingStatus": "Partial success",
        "batchReleaseStatus": "Not released",
        "totalCount": 6,
        "totalErrors": 5,
        "totalSuccess": 1,
        "totalPublic": 0,
        "submissions": [
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {"field": "ConditionID.type", "value": "MONDO"},
                            {"field": "ConditionID.value", "value": "MONDO:0009861"},
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "The identifier you provided (MONDO:MONDO:0009861) cannot be validated, either because it is invalid or because it is valid but not in our database yet. For the latter case, contact us at clinvar@ncbi.nlm.nih.gov."
                                }
                            ]
                        },
                    }
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {"field": "ConditionID.type", "value": "MONDO"},
                            {"field": "ConditionID.value", "value": "MONDO:000"},
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "The identifier you provided (MONDO:MONDO:000) cannot be validated, either because it is invalid or because it is valid but not in our database yet. For the latter case, contact us at clinvar@ncbi.nlm.nih.gov."
                                }
                            ]
                        },
                    },
                    {
                        "input": [],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "Records adefc5ed-7d59-4119-8b3d-07dcdc504c09, adefc5ed-7d59-4119-8b3d-07dcdc504c09 and adefc5ed-7d59-4119-8b3d-07dcdc504c09 report the same variant/condition pair. Please remove all but one of these records. If your intent is to provide data for more than one individuals with the variant, please report them as distinct observations (ObservedIn)."
                                }
                            ]
                        },
                    },
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {"field": "ConditionID.type", "value": "MONDO"},
                            {"field": "ConditionID.value", "value": "MONDO:000"},
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "The identifier you provided (MONDO:MONDO:000) cannot be validated, either because it is invalid or because it is valid but not in our database yet. For the latter case, contact us at clinvar@ncbi.nlm.nih.gov."
                                }
                            ]
                        },
                    },
                    {
                        "input": [],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "Records adefc5ed-7d59-4119-8b3d-07dcdc504c09, adefc5ed-7d59-4119-8b3d-07dcdc504c09 and adefc5ed-7d59-4119-8b3d-07dcdc504c09 report the same variant/condition pair. Please remove all but one of these records. If your intent is to provide data for more than one individuals with the variant, please report them as distinct observations (ObservedIn)."
                                }
                            ]
                        },
                    },
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {"field": "ConditionID.type", "value": "MONDO"},
                            {"field": "ConditionID.value", "value": "MONDO:000"},
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "The identifier you provided (MONDO:MONDO:000) cannot be validated, either because it is invalid or because it is valid but not in our database yet. For the latter case, contact us at clinvar@ncbi.nlm.nih.gov."
                                }
                            ]
                        },
                    },
                    {
                        "input": [],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "Records adefc5ed-7d59-4119-8b3d-07dcdc504c09, adefc5ed-7d59-4119-8b3d-07dcdc504c09 and adefc5ed-7d59-4119-8b3d-07dcdc504c09 report the same variant/condition pair. Please remove all but one of these records. If your intent is to provide data for more than one individuals with the variant, please report them as distinct observations (ObservedIn)."
                                }
                            ]
                        },
                    },
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "clinvarAccession": "SCV000839746",
                },
                "processingStatus": "Success",
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "localKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [{"value": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2"}],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "You provided multiple rows on the Variant tab with the same LinkingID adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2."
                                }
                            ]
                        },
                    }
                ],
            },
        ],
    }


@pytest.fixture
def data_summary_response_error_all():
    """Example of a submission with a status of error where all records failed:

    This response contains a URL for a summary file with more details. Following is an example:
    """
    return {
        "submissionName": "PAHVCEP_10_2020_API",
        "submissionDate": "2021-03-25",
        "batchProcessingStatus": "Error",
        "batchReleaseStatus": "Not released",
        "totalCount": 3,
        "totalErrors": 3,
        "totalSuccess": 0,
        "totalPublic": 0,
        "submissions": [
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1|MedGen_C0149978",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {
                                "value": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1|MedGen_C0149978"
                            }
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "This assertion duplicates another assertion with the RecordID adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1, ClinVar accession SCV000839746, and Submitted variant NC_000012.12:g.102852945T>M."
                                }
                            ]
                        },
                    }
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2|MedGen_C0153436",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {
                                "value": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2|MedGen_C0153436"
                            }
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "This assertion duplicates another assertion with the RecordID adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2, ClinVar accession SCV000839747, and Submitted variant NC_000012.12:g.102852945T>M."
                                }
                            ]
                        },
                    }
                ],
            },
            {
                "identifiers": {
                    "localID": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3",
                    "clinvarLocalKey": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3|MedGen_C1134719",
                },
                "processingStatus": "Error",
                "errors": [
                    {
                        "input": [
                            {
                                "value": "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3|MedGen_C1134719"
                            }
                        ],
                        "output": {
                            "errors": [
                                {
                                    "userMessage": "This assertion duplicates another assertion with the RecordID adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3, ClinVar accession SCV000839748, and Submitted variant NC_000012.12:g.102852945T>M."
                                }
                            ]
                        },
                    }
                ],
            },
        ],
    }


@pytest.fixture
def data_submission_snv():
    return {
        "clinvarSubmissionReleaseStatus": "hold until published",
        "clinvarSubmission": [
            {
                "assertionCriteria": {
                    "citation": {"db": "PubMed", "id": "25741868"},
                    "method": "ACMG",
                },
                "clinicalSignificance": {
                    "citation": [],
                    "clinicalSignificanceDescription": "Likely pathogenic",
                    "comment": "PM1_sup, PP3_mod, PM2_sup, PP4_mod",
                    "dateLastEvaluated": "2022-11-09",
                    "modeOfInheritance": "Autosomal recessive inheritance",
                },
                "conditionSet": {"condition": [{"name": "not provided"}]},
                "observedIn": [
                    {
                        "affectedStatus": "yes",
                        "alleleOrigin": "biparental",
                        "clinicalFeatures": [
                            {
                                "db": "HP",
                                "id": "HP:0003097",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0011427",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0002643",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0002089",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0005180",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0001762",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0001342",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                            {
                                "db": "HP",
                                "id": "HP:0001250",
                                "clinicalFeaturesAffectedStatus": "present",
                            },
                        ],
                        "numberOfIndividuals": 1,
                        "collectionMethod": "clinical testing",
                    }
                ],
                "recordStatus": "novel",
                "variantSet": {
                    "variant": [
                        {
                            "chromosomeCoordinates": {
                                "assembly": "GRCh37",
                                "chromosome": "9",
                                "start": 116021007,
                                "stop": 116021007,
                                "referenceAllele": "T",
                                "alternateAllele": "C",
                            }
                        }
                    ]
                },
            }
        ],
    }



from clinvar_api import common, models, msg


def test_from_msg_data_created(data_created):
    msg_record = common.CONVERTER.structure(data_created, msg.Created)
    assert models.Created.from_msg(msg_record)


def test_from_msg_data_message(data_message):
    msg_record = common.CONVERTER.structure(data_message, msg.Error)
    assert models.Error.from_msg(msg_record)


def test_from_msg_data_submission_submitted(data_submission_submitted):
    msg_record = common.CONVERTER.structure(data_submission_submitted, msg.SubmissionStatus)
    assert models.SubmissionStatus.from_msg(msg_record)


def test_from_msg_data_submission_processing(data_submission_processing):
    msg_record = common.CONVERTER.structure(data_submission_processing, msg.SubmissionStatus)
    assert models.SubmissionStatus.from_msg(msg_record)


def test_from_msg_data_submission_processed(data_submission_processed):
    msg_record = common.CONVERTER.structure(data_submission_processed, msg.SubmissionStatus)
    assert models.SubmissionStatus.from_msg(msg_record)


def test_from_msg_data_partially_successful_submission(data_partially_successful_submission):
    msg_record = common.CONVERTER.structure(
        data_partially_successful_submission, msg.SubmissionStatus
    )
    assert models.SubmissionStatus.from_msg(msg_record)



import datetime
import json

import cattrs
import pytest

from clinvar_data import conversion, models


def test_force_list_lst():
    assert models.force_list(1) == [1]
    assert models.force_list([1]) == [1]


@pytest.mark.parametrize(
    "input,expected",
    [
        (
            {"@DB": "MedGen", "@ID": "C1704981"},
            {"db": "MedGen", "id": "C1704981"},
        ),
        (
            {
                "@DB": "MedGen",
                "@ID": "C1704981",
                "@Type": "CUI",
                "@URL": "about:blank",
                "@Status": "current",
            },
            {
                "db": "MedGen",
                "id": "C1704981",
                "type": "CUI",
                "url": "about:blank",
                "status": "current",
            },
        ),
    ],
)
def test_xref_type_from_json_data(input, expected):
    obj = models.Xref.from_json_data(input)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        ("affects", models.ClinicalSignificanceDescription.AFFECTS),
        ("x", models.ClinicalSignificanceDescription.OTHER),
    ],
)
def test_clinical_significance_description_from_the_wild(value, expected):
    value = models.ClinicalSignificanceDescription.from_the_wild(value)
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        ("comment text", {"text": "comment text"}),
        ({"#text": "comment text"}, {"text": "comment text"}),
        (
            {"@DataSource": "NCBI curation", "@Type": "public", "#text": "comment text"},
            {"text": "comment text", "type": "public", "datasource": "NCBI curation"},
        ),
    ],
)
def test_comment_from_json_data(value, expected):
    obj = models.Comment.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        ({"@Source": "PubMed", "#text": "9731074"}, {"source": "PubMed", "value": "9731074"}),
        ({"@Source": "PubMed", "#text": "123"}, {"source": "PubMed", "value": "123"}),
    ],
)
def test_citation_identifier_from_json_data(value, expected):
    obj = models.CitationIdentifier.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Type": "general", "ID": {"@Source": "PubMed", "#text": "8547665"}},
            {"ids": [{"source": "PubMed", "value": "8547665"}], "type": "general"},
        ),
        ({"CitationText": "text here"}, {"citation_text": "text here"}),
        (
            {
                "@Type": "review",
                "@Abbrev": "GeneReviews",
                "ID": [
                    {"@Source": "PubMed", "#text": "29595935"},
                    {"@Source": "BookShelf", "#text": "NBK488189"},
                ],
            },
            {
                "abbrev": "GeneReviews",
                "ids": [
                    {"source": "PubMed", "value": "29595935"},
                    {"source": "BookShelf", "value": "NBK488189"},
                ],
                "type": "review",
            },
        ),
        (
            {"URL": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=NEFL"},
            {"url": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=NEFL"},
        ),
    ],
)
def test_citation_from_json_data(value, expected):
    obj = models.Citation.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@type": "ACMG/ClinGen CNV Guidelines, 2019", "#text": "3"},
            {"type": "ACMG/ClinGen CNV Guidelines, 2019", "value": 3.0},
        ),
        ({"@type": "Geneting testing", "#text": "5"}, {"type": "Geneting testing", "value": 5.0}),
        (
            {"@type": "ACMG/ClinGen CNV Guidelines, 2019", "#text": "0.95"},
            {"type": "ACMG/ClinGen CNV Guidelines, 2019", "value": 0.95},
        ),
    ],
)
def test_custom_assertion_score_from_json_data(value, expected):
    obj = models.CustomAssertionScore.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@DateLastEvaluated": "2010-09-09",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Pathogenic",
            },
            {
                "date_last_evaluated": datetime.date(2010, 9, 9),
                "descriptions": ["pathogenic"],
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2020-01-06",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Benign",
                "Comment": "comment text",
            },
            {
                "comments": [{"text": "comment text"}],
                "date_last_evaluated": datetime.date(2020, 1, 6),
                "descriptions": ["benign"],
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2020-04-16",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Pathogenic",
                "Citation": {"@Type": "general", "ID": {"@Source": "PubMed", "#text": "32576985"}},
            },
            {
                "citations": [
                    {"ids": [{"source": "PubMed", "value": "32576985"}], "type": "general"}
                ],
                "date_last_evaluated": datetime.date(2020, 4, 16),
                "descriptions": ["pathogenic"],
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2016-05-01",
                "ReviewStatus": "no assertion criteria provided",
                "Description": ["other", "other"],
                "ExplanationOfInterpretation": "explanation text",
            },
            {
                "date_last_evaluated": datetime.date(2016, 5, 1),
                "descriptions": ["other", "other"],
                "explanation_of_interpretation": "explanation text",
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2022-09-27",
                "ReviewStatus": {
                    "@xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
                    "#text": "criteria provided, single submitter",
                },
                "Description": [
                    {
                        "@xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
                        "#text": "Pathogenic",
                    },
                    {
                        "@xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
                        "#text": "Pathogenic",
                    },
                ],
            },
            {
                "date_last_evaluated": datetime.date(2022, 9, 27),
                "descriptions": ["pathogenic", "pathogenic"],
                "review_status": "criteria provided, single submitter",
            },
        ),
    ],
)
def test_clinical_significance_scv_from_json_data(value, expected):
    obj = models.ClinicalSignificanceTypeSCV.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@DateLastEvaluated": "2010-09-09",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Pathogenic",
            },
            {
                "date_last_evaluated": datetime.date(2010, 9, 9),
                "description": "pathogenic",
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2020-01-06",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Benign",
                "Comment": "comment text",
            },
            {
                "comments": [{"text": "comment text"}],
                "date_last_evaluated": datetime.date(2020, 1, 6),
                "description": "benign",
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2020-04-16",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Pathogenic",
                "Citation": {"@Type": "general", "ID": {"@Source": "PubMed", "#text": "32576985"}},
            },
            {
                "citations": [
                    {"ids": [{"source": "PubMed", "value": "32576985"}], "type": "general"}
                ],
                "date_last_evaluated": datetime.date(2020, 4, 16),
                "description": "pathogenic",
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2016-05-01",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "other",
                "ExplanationOfInterpretation": "explanation text",
            },
            {
                "date_last_evaluated": datetime.date(2016, 5, 1),
                "description": "other",
                "explanation_of_interpretation": "explanation text",
                "review_status": "no assertion criteria provided",
            },
        ),
        (
            {
                "@DateLastEvaluated": "2022-09-27",
                "ReviewStatus": {
                    "@xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
                    "#text": "criteria provided, single submitter",
                },
                "Description": {
                    "@xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
                    "#text": "Pathogenic",
                },
            },
            {
                "date_last_evaluated": datetime.date(2022, 9, 27),
                "description": "pathogenic",
                "review_status": "criteria provided, single submitter",
            },
        ),
    ],
)
def test_clinical_significance_rcv_from_json_data(value, expected):
    obj = models.ClinicalSignificanceRCV.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Acc": "RCV000000009",
                "@DateUpdated": "2022-04-23",
                "@DateCreated": "2013-04-04",
                "@Version": "4",
                "@Type": "RCV",
            },
            {
                "acc": "RCV000000009",
                "date_created": datetime.date(2013, 4, 4),
                "date_updated": datetime.date(2022, 4, 23),
                "type": "RCV",
                "version": 4,
            },
        ),
    ],
)
def test_reference_clinvar_accession_from_json_data(value, expected):
    obj = models.ReferenceClinVarAccession.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Attribute": {
                    "@Type": "ModeOfInheritance",
                    "@integerValue": "263",
                    "#text": "Autosomal recessive inheritance",
                }
            },
            {
                "integer_value": 263,
                "type": "ModeOfInheritance",
                "value": "Autosomal recessive inheritance",
            },
        ),
        (
            {
                "Attribute": {
                    "@Type": "ModeOfInheritance",
                    "@integerValue": "263",
                    "#text": "Autosomal recessive inheritance",
                },
                "XRef": {
                    "@ID": "FGDYS-ID3",
                    "@DB": "Laboratory of Molecular ...",
                },
            },
            {
                "integer_value": 263,
                "type": "ModeOfInheritance",
                "value": "Autosomal recessive inheritance",
                "xrefs": [{"db": "Laboratory of Molecular ...", "id": "FGDYS-ID3"}],
            },
        ),
    ],
)
def test_clinvar_accession_from_json_data(value, expected):
    obj = models.ReferenceClinVarAssertionAttribute.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"Name": "ABCC8:c.2117-3C>T", "Zygosity": "SingleHeterozygote"},
            {"name": "ABCC8:c.2117-3C>T", "zygosity": "SingleHeterozygote"},
        ),
        (
            {
                "Name": "ABCC8:c.2117-3C>T",
                "Zygosity": "SingleHeterozygote",
                "RelativeOrientation": "cis",
            },
            {
                "name": "ABCC8:c.2117-3C>T",
                "relative_orientation": "cis",
                "zygosity": "SingleHeterozygote",
            },
        ),
    ],
)
def test_allele_description_from_json_data(value, expected):
    obj = models.AlleleDescription.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Zygosity": "SingleHeterozygote",
                "AlleleDescSet": [
                    {"Name": "ABCC8:c.2117-3C>T", "Zygosity": "SingleHeterozygote"},
                    {"Name": "KCNJ11:c.1009G>A", "Zygosity": "Homozygote"},
                ],
                "Count": "1",
            },
            {
                "allele_descriptions": [
                    {"name": "ABCC8:c.2117-3C>T", "zygosity": "SingleHeterozygote"},
                    {"name": "KCNJ11:c.1009G>A", "zygosity": "Homozygote"},
                ],
                "count": 1,
                "zygosity": "SingleHeterozygote",
            },
        ),
        (
            {
                "Zygosity": "SingleHeterozygote",
                "AlleleDescSet": [],
            },
            {"zygosity": "SingleHeterozygote"},
        ),
        (
            {},
            None,
        ),
    ],
)
def test_cooccurrence_from_json_data(value, expected):
    obj = models.Cooccurrence.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        ("curation", models.ObservationMethod.CURATION),
        ("x", models.ObservationMethod.OTHER),
    ],
)
def test_observation_method_from_the_wild(value, expected):
    value = models.ObservationMethod.from_the_wild(value)
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Type": "Description", "#text": "Tournamille et al. (1998) ..."},
            {"type": "Description", "value": "Tournamille et al. (1998) ..."},
        ),
    ],
)
def test_observed_data_attribute_from_json_data(value, expected):
    obj = models.ObservedDataAttribute.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"Attribute": {"@Type": "Description", "#text": "not provided"}},
            {"attribute": {"type": "Description", "value": "not provided"}},
        ),
        (
            {
                "Attribute": {
                    "@Type": "Description",
                    "#text": "Tournamille et al. (1998) and Olsson et al. (1998)...",
                },
                "Citation": [
                    {"ID": {"@Source": "PubMed", "#text": "9731074"}},
                    {"ID": {"@Source": "PubMed", "#text": "9886340"}},
                ],
            },
            {
                "attribute": {
                    "type": "Description",
                    "value": "Tournamille et al. (1998) and Olsson et al. " "(1998)...",
                },
                "citations": [
                    {"ids": [{"source": "PubMed", "value": "9731074"}]},
                    {"ids": [{"source": "PubMed", "value": "9886340"}]},
                ],
            },
        ),
        (
            {
                "Attribute": {
                    "@Type": "Description",
                    "#text": "In a patient with chronic hemolytic...",
                },
                "Citation": {"ID": {"@Source": "PubMed", "#text": "6933565"}},
                "XRef": {"@DB": "OMIM", "@ID": "300653", "@Type": "MIM"},
            },
            {
                "attribute": {
                    "type": "Description",
                    "value": "In a patient with chronic hemolytic...",
                },
                "citations": [{"ids": [{"source": "PubMed", "value": "6933565"}]}],
                "xrefs": [{"db": "OMIM", "id": "300653", "type": "MIM"}],
            },
        ),
    ],
)
def test_observed_data_from_json_data(value, expected):
    obj = models.ObservedData.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Description": {
                    "@Type": "public",
                    "#text": "Phenotype: fine and gross motor delay...",
                }
            },
            {"description": {"text": "Phenotype: fine and gross motor delay...", "type": "public"}},
        ),
    ],
)
def test_sample_description_from_json_data(value, expected):
    obj = models.SampleDescription.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@NumFamiliesWithSegregationObserved": "1",
                "@NumFamiliesWithVariant": "1",
                "FamilyHistory": "yes",
            },
            {
                "family_history": "yes",
                "num_families_with_segregation_observed": 1,
                "num_families_with_variant": 1,
            },
        ),
        (
            {"@NumFamiliesWithVariant": "1", "FamilyHistory": "no"},
            {"family_history": "no", "num_families_with_variant": 1},
        ),
    ],
)
def test_family_info_from_json_data(value, expected):
    obj = models.FamilyInfo.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@TaxonomyId": "9606", "#text": "human"},
            {"taxonomy_id": 9606, "value": "human"},
        ),
        (
            "human",
            {"value": "human"},
        ),
    ],
)
def test_species_from_json_data(value, expected):
    obj = models.Species.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Type": "minimum", "@age_unit": "years", "#text": "10"},
            {"age_unit": "years", "type": "minimum", "value": 10},
        ),
        (
            {"@Type": "maximum", "@age_unit": "years", "#text": "69"},
            {"age_unit": "years", "type": "maximum", "value": 69},
        ),
        (
            {"@Type": "single", "@age_unit": "years", "#text": "80"},
            {"age_unit": "years", "type": "single", "value": 80},
        ),
    ],
)
def test_age_from_json_data(value, expected):
    obj = models.Age.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Type": "Preferred", "#text": "ACKR1"},
            {"type": "Preferred", "value": "ACKR1"},
        ),
        (
            {"@Type": "Preferred", "#text": "NM_002036.4(ACKR1):c.265C>T (p.Arg89Cys)"},
            {"type": "Preferred", "value": "NM_002036.4(ACKR1):c.265C>T (p.Arg89Cys)"},
        ),
        (
            {"@Type": "Alternate", "#text": "NM_001127505.2:c.155A>G(p.Tyr52Cys)"},
            {"type": "Alternate", "value": "NM_001127505.2:c.155A>G(p.Tyr52Cys)"},
        ),
    ],
)
def test_typed_value_from_json_data(value, expected):
    obj = models.TypedValue.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "ElementValue": {
                    "@Type": "Preferred",
                    "#text": "NM_006642.5(SDCCAG8):c.1946_1949del (p.Cys649fs)",
                },
                "XRef": {"@ID": "CA113828", "@DB": "ClinGen"},
            },
            {
                "value": {
                    "type": "Preferred",
                    "value": "NM_006642.5(SDCCAG8):c.1946_1949del (p.Cys649fs)",
                },
                "xrefs": [{"db": "ClinGen", "id": "CA113828"}],
            },
        ),
    ],
)
def test_annotated_typed_value_from_json_data(value, expected):
    obj = models.AnnotatedTypedValue.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@DataSource": "MedGen", "@ID": "CN192494"},
            {"data_source": "MedGen", "id": "CN192494"},
        ),
        (
            {"@DataSource": "MedGen", "@ID": "CN192494", "@SourceType": "resource"},
            {"data_source": "MedGen", "id": "CN192494", "source_type": "resource"},
        ),
    ],
)
def test_source_from_json_data(value, expected):
    obj = models.Source.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Type": "co-occurring condition", "@ID": "70"},
            {"id": 70, "type": "co-occurring condition"},
        ),
        (
            {
                "@Type": "DrugResponseAndDisease",
                "Name": {"ElementValue": {"@Type": "Preferred", "#text": "Breast Cancer"}},
            },
            {
                "names": [{"value": {"type": "Preferred", "value": "Breast Cancer"}}],
                "type": "DrugResponseAndDisease",
            },
        ),
        (
            {"@Type": "Finding member", "@ID": "890"},
            {"id": 890, "type": "Finding member"},
        ),
    ],
)
def test_trait_relationship_from_json_data(value, expected):
    obj = models.TraitRelationship.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"Name": {"ElementValue": {"@Type": "Preferred", "#text": "BARDET-BIEDL SYNDROME 16"}}},
            {"names": [{"value": {"type": "Preferred", "value": "BARDET-BIEDL SYNDROME 16"}}]},
        ),
    ],
)
def test_clinvar_assertion_trait_relationship_from_json_data(value, expected):
    obj = models.ClinVarAssertionTraitRelationship.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"Name": {"ElementValue": {"@Type": "Preferred", "#text": "BARDET-BIEDL SYNDROME 16"}}},
            {"names": [{"value": {"type": "Preferred", "value": "BARDET-BIEDL SYNDROME 16"}}]},
        ),
    ],
)
def test_clinvar_assertion_trait_from_json_data(value, expected):
    obj = models.ClinVarAssertionTrait.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@ID": "15693",
                "@Type": "BloodGroup",
                "Name": {
                    "ElementValue": {
                        "@Type": "Preferred",
                        "#text": "DUFFY BLOOD GROUP SYSTEM, FY(bwk) PHENOTYPE",
                    },
                    "XRef": {"@Type": "Allelic variant", "@ID": "613665.0003", "@DB": "OMIM"},
                },
            },
            {
                "names": [
                    {
                        "value": {
                            "type": "Preferred",
                            "value": "DUFFY BLOOD GROUP SYSTEM, FY(bwk) PHENOTYPE",
                        },
                        "xrefs": [{"db": "OMIM", "id": "613665.0003", "type": "Allelic variant"}],
                    }
                ],
                "type": "BloodGroup",
            },
        ),
    ],
)
def test_trait_from_json_data(value, expected):
    obj = models.Trait.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Type": "Indication",
                "Trait": {
                    "@Type": "Finding",
                    "Name": {"ElementValue": {"@Type": "Preferred", "#text": "Diagnostic"}},
                },
            },
            {
                "traits": [{"names": [{"value": {"type": "Preferred", "value": "Diagnostic"}}]}],
                "type": "Indication",
            },
        ),
        (
            {
                "@Type": "Indication",
                "Trait": {
                    "@Type": "Finding",
                    "Name": {"ElementValue": {"@Type": "Preferred", "#text": "Vision phenotype"}},
                },
            },
            {
                "traits": [
                    {"names": [{"value": {"type": "Preferred", "value": "Vision phenotype"}}]}
                ],
                "type": "Indication",
            },
        ),
    ],
)
def test_indication_from_json_data(value, expected):
    obj = models.Indication.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Origin": "germline",
                "Species": {"@TaxonomyId": "9606", "#text": "human"},
                "AffectedStatus": "not provided",
            },
            {
                "affected_status": "not provided",
                "origin": "germline",
                "species": {"taxonomy_id": 9606, "value": "human"},
            },
        ),
        (
            {
                "Origin": "unknown",
                "Species": {"@TaxonomyId": "9606", "#text": "human"},
                "AffectedStatus": "not provided",
                "NumberTested": "3",
                "NumberMales": "1",
                "FamilyData": {"@NumFamilies": "1"},
            },
            {
                "affected_status": "not provided",
                "family_data": {"num_families": 1},
                "number_males": 1,
                "number_tested": 3,
                "origin": "unknown",
                "species": {"taxonomy_id": 9606, "value": "human"},
            },
        ),
    ],
)
def test_sample_from_json_data(value, expected):
    obj = models.Sample.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Sample": {
                    "Origin": "germline",
                    "Species": "human",
                    "AffectedStatus": "not provided",
                },
                "Method": {"MethodType": "literature only"},
                "ObservedData": [
                    {
                        "Attribute": {"@Type": "Description", "#text": "In a 27-week-old male ..."},
                        "Citation": {"ID": {"@Source": "PubMed", "#text": "19465910"}},
                        "XRef": {"@DB": "OMIM", "@ID": "610031", "@Type": "MIM"},
                    },
                    {
                        "Attribute": {
                            "@Type": "Description",
                            "#text": "Fallet-Bianco et al. (2014)...",
                        },
                        "Citation": {"ID": {"@Source": "PubMed", "#text": "25059107"}},
                    },
                ],
            },
            {
                "methods": ["literature only"],
                "observed_data": [
                    {
                        "attribute": {"type": "Description", "value": "In a 27-week-old male ..."},
                        "citations": [{"ids": [{"source": "PubMed", "value": "19465910"}]}],
                        "xrefs": [{"db": "OMIM", "id": "610031", "type": "MIM"}],
                    },
                    {
                        "attribute": {
                            "type": "Description",
                            "value": "Fallet-Bianco et al. (2014)...",
                        },
                        "citations": [{"ids": [{"source": "PubMed", "value": "25059107"}]}],
                    },
                ],
                "sample": {
                    "affected_status": "not provided",
                    "origin": "germline",
                    "species": {"value": "human"},
                },
            },
        ),
        (
            {
                "Sample": {
                    "Origin": "germline",
                    "Species": {"@TaxonomyId": "9606", "#text": "human"},
                    "AffectedStatus": "unknown",
                    "NumberTested": "1",
                },
                "Method": {"MethodType": "clinical testing"},
                "ObservedData": {
                    "@ID": "110387662",
                    "Attribute": {"@integerValue": "1", "@Type": "VariantAlleles"},
                },
            },
            {
                "methods": ["clinical testing"],
                "observed_data": [{"attribute": {"integer_value": 1, "type": "VariantAlleles"}}],
                "sample": {
                    "affected_status": "unknown",
                    "number_tested": 1,
                    "origin": "germline",
                    "species": {"taxonomy_id": 9606, "value": "human"},
                },
            },
        ),
    ],
)
def test_observation_set_from_json_data(value, expected):
    obj = models.ObservationSet.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Attribute": {
                    "@Accession": "NM_170784",
                    "@Version": "3",
                    "@Change": "c.281del",
                    "@Type": "HGVS, coding, RefSeq",
                    "@MANESelect": "true",
                    "#text": "NM_170784.3:c.281del",
                }
            },
            {"change": "c.281del", "type": "HGVS, coding, RefSeq", "value": "NM_170784.3:c.281del"},
        ),
        (
            {
                "Attribute": {
                    "@Accession": "NM_170784",
                    "@Version": "2",
                    "@Change": "c.281del",
                    "@Type": "HGVS, previous",
                    "#text": "NM_170784.2:c.281del",
                }
            },
            {"change": "c.281del", "type": "HGVS, previous", "value": "NM_170784.2:c.281del"},
        ),
        (
            {
                "Attribute": {"@Type": "MolecularConsequence", "#text": "frameshift variant"},
                "XRef": [
                    {"@ID": "SO:0001589", "@DB": "Sequence Ontology"},
                    {"@ID": "NM_018848.3:c.281del", "@DB": "RefSeq"},
                ],
            },
            {
                "type": "MolecularConsequence",
                "value": "frameshift variant",
                "xrefs": [
                    {"db": "Sequence Ontology", "id": "SO:0001589"},
                    {"db": "RefSeq", "id": "NM_018848.3:c.281del"},
                ],
            },
        ),
    ],
)
def test_measure_set_attribute_from_json_data(value, expected):
    obj = models.MeasureSetAttribute.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Attribute": {
                    "@Accession": "LRG_801t1",
                    "@Type": "HGVS, coding, LRG",
                    "#text": "LRG_801t1:c.265C>T",
                }
            },
            {"type": "HGVS, coding, LRG", "value": "LRG_801t1:c.265C>T"},
        ),
        (
            {
                "Attribute": {
                    "@Accession": "NM_002036",
                    "@Version": "4",
                    "@Change": "c.286_299del",
                    "@Type": "HGVS, coding, RefSeq",
                    "@MANESelect": "true",
                    "#text": "NM_002036.4:c.286_299del",
                }
            },
            {"type": "HGVS, coding, RefSeq", "value": "NM_002036.4:c.286_299del"},
        ),
    ],
)
def test_measure_type_attribute_from_json_data(value, expected):
    obj = models.MeasureAttribute.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Value": "0.01099",
                "@Source": "NHLBI Exome Sequencing Project (ESP) Exome Variant Server",
            },
            {
                "source": "NHLBI Exome Sequencing Project (ESP) Exome Variant Server",
                "value": 0.01099,
            },
        ),
        (
            {"@Value": "0.00001", "@Source": "Exome Aggregation Consortium (ExAC)"},
            {"source": "Exome Aggregation Consortium (ExAC)", "value": 1e-05},
        ),
    ],
)
def test_allele_frequency_from_json_data(value, expected):
    obj = models.AlleleFrequency.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {"@Value": "0.00459", "@Source": "1000 Genomes Project", "@MinorAllele": "T"},
            {"minor_allele": "T", "source": "1000 Genomes Project", "value": 0.00459},
        ),
        (
            {"@Value": "0.00080", "@Source": "1000 Genomes Project", "@MinorAllele": "A"},
            {"minor_allele": "A", "source": "1000 Genomes Project", "value": 0.0008},
        ),
    ],
)
def test_global_minor_allele_frequency_from_json_data(value, expected):
    obj = models.GlobalMinorAlleleFrequency.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Acc": "SCV000020152",
                "@DateCreated": "2013-04-04",
                "@DateUpdated": "2017-12-15",
                "@Version": "3",
                "@Type": "SCV",
                "@OrgID": "3",
                "@OrganizationCategory": "resource",
                "@OrgType": "primary",
            },
            {
                "acc": "SCV000020152",
                "date_created": datetime.date(2013, 4, 4),
                "date_updated": datetime.date(2017, 12, 15),
                "org_category": "resource",
                "org_id": "3",
                "org_type": "primary",
                "type": "SCV",
                "version": 3,
            },
        ),
    ],
)
def test_clinvar_assertion_accession_from_json_data(value, expected):
    obj = models.ClinVarAssertionAccession.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Assembly": "GRCh38",
                "@AssemblyAccessionVersion": "GCF_000001405.38",
                "@AssemblyStatus": "current",
                "@Chr": "16",
                "@Accession": "NC_000016.10",
                "@start": "50742086",
                "@stop": "50801935",
                "@display_start": "50742086",
                "@display_stop": "50801935",
                "@Strand": "+",
            },
            {
                "accession": "NC_000016.10",
                "assembly": "GRCh38",
                "assembly_accesion_version": "GCF_000001405.38",
                "assembly_status": "current",
                "chr": "16",
                "display_start": 50742086,
                "display_stop": 50801935,
                "start": 50742086,
                "stop": 50801935,
                "strand": "+",
            },
        ),
        (
            {
                "@Assembly": "GRCh37",
                "@AssemblyAccessionVersion": "GCF_000001405.25",
                "@AssemblyStatus": "previous",
                "@Chr": "6",
                "@Accession": "NC_000006.11",
                "@start": "41129260",
                "@stop": "41129260",
                "@display_start": "41129260",
                "@display_stop": "41129260",
                "@variantLength": "1",
                "@positionVCF": "41129260",
                "@referenceAlleleVCF": "C",
                "@alternateAlleleVCF": "T",
            },
            {
                "accession": "NC_000006.11",
                "alternate_allele_vcf": "T",
                "assembly": "GRCh37",
                "assembly_accesion_version": "GCF_000001405.25",
                "assembly_status": "previous",
                "chr": "6",
                "display_start": 41129260,
                "display_stop": 41129260,
                "position_vcf": 41129260,
                "reference_allele_vcf": "C",
                "start": 41129260,
                "stop": 41129260,
                "variant_length": 1,
            },
        ),
    ],
)
def test_sequence_location_from_json_data(value, expected):
    obj = models.SequenceLocation.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "Attribute": {
                    "@dateValue": "2020-05-27",
                    "@Type": "Haploinsufficiency",
                    "#text": "Little evidence for dosage pathogenicity",
                },
                "Citation": {
                    "URL": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                },
            },
            {
                "citations": [
                    {
                        "url": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                    }
                ],
                "date_value": datetime.datetime(2020, 5, 27, 0, 0),
                "type": "Haploinsufficiency",
                "value": "Little evidence for dosage pathogenicity",
            },
        ),
        (
            {
                "Attribute": {
                    "@dateValue": "2020-05-27",
                    "@Type": "Triplosensitivity",
                    "#text": "No evidence available",
                },
                "Citation": {
                    "URL": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                },
            },
            {
                "citations": [
                    {
                        "url": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                    }
                ],
                "date_value": datetime.datetime(2020, 5, 27, 0, 0),
                "type": "Triplosensitivity",
                "value": "No evidence available",
            },
        ),
    ],
)
def test_measure_relationship_attribute_from_json_data(value, expected):
    obj = models.MeasureRelationshipAttribute.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value",
    [
        {
            "@Type": "within single gene",
            "Name": {
                "ElementValue": {
                    "@Type": "Preferred",
                    "#text": "TGF-beta activated kinase 1 (MAP3K7) binding protein 2",
                }
            },
            "Symbol": {"ElementValue": {"@Type": "Preferred", "#text": "TAB2"}},
            "AttributeSet": [
                {
                    "Attribute": {
                        "@dateValue": "2020-05-27",
                        "@Type": "Haploinsufficiency",
                        "#text": "Little evidence for dosage pathogenicity",
                    },
                    "Citation": {
                        "URL": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                    },
                },
                {
                    "Attribute": {
                        "@dateValue": "2020-05-27",
                        "@Type": "Triplosensitivity",
                        "#text": "No evidence available",
                    },
                    "Citation": {
                        "URL": "https://www.ncbi.nlm.nih.gov/projects/dbvar/ISCA/isca_gene.cgi?sym=TAB2"
                    },
                },
            ],
            "SequenceLocation": [
                {
                    "@Assembly": "GRCh38",
                    "@AssemblyAccessionVersion": "GCF_000001405.38",
                    "@AssemblyStatus": "current",
                    "@Chr": "6",
                    "@Accession": "NC_000006.12",
                    "@start": "149217926",
                    "@stop": "149411607",
                    "@display_start": "149217926",
                    "@display_stop": "149411607",
                    "@Strand": "+",
                },
            ],
            "XRef": [
                {"@ID": "23118", "@DB": "Gene"},
            ],
        },
    ],
)
def test_measure_relationship_from_json_data(value, snapshot):
    obj = models.MeasureRelationship.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    # NB: the input/expect output value pairs do not fit on one screen any more for
    # this test, so we use snapshot testing
    snapshot.assert_match(json.dumps(value, indent=2), "output")


@pytest.mark.parametrize(
    "value",
    [
        {
            "@Type": "Deletion",
            "@ID": "33436",
            "Name": [
                {
                    "ElementValue": {
                        "@Type": "Preferred",
                        "#text": "NM_002036.4(ACKR1):c.286_299del (p.Trp96fs)",
                    }
                },
                {
                    "ElementValue": {"@Type": "Alternate", "#text": "ACKR1, 14-BP DEL, NT286"},
                    "XRef": {"@Type": "Allelic variant", "@ID": "613665.0004", "@DB": "OMIM"},
                },
            ],
            "CanonicalSPDI": "NC_000001.11:159205718:CCTGGCTGGCCTGTCCTGGC:CCTGGC",
            "AttributeSet": [
                {
                    "Attribute": {
                        "@Accession": "LRG_801t1",
                        "@Type": "HGVS, coding, LRG",
                        "#text": "LRG_801t1:c.286_299del",
                    }
                },
            ],
            "CytogeneticLocation": "1q23.2",
            "SequenceLocation": [
                {
                    "@Assembly": "GRCh38",
                    "@AssemblyAccessionVersion": "GCF_000001405.38",
                    "@AssemblyStatus": "current",
                    "@Chr": "1",
                    "@Accession": "NC_000001.11",
                    "@start": "159205719",
                    "@stop": "159205732",
                    "@display_start": "159205719",
                    "@display_stop": "159205732",
                    "@variantLength": "14",
                    "@positionVCF": "159205718",
                    "@referenceAlleleVCF": "CCCTGGCTGGCCTGT",
                    "@alternateAlleleVCF": "C",
                },
            ],
            "MeasureRelationship": {
                "@Type": "within single gene",
                "Name": {
                    "ElementValue": {
                        "@Type": "Preferred",
                        "#text": "atypical chemokine receptor 1 (Duffy blood group)",
                    }
                },
                "Symbol": {"ElementValue": {"@Type": "Preferred", "#text": "ACKR1"}},
                "SequenceLocation": [
                    {
                        "@Assembly": "GRCh38",
                        "@AssemblyAccessionVersion": "GCF_000001405.38",
                        "@AssemblyStatus": "current",
                        "@Chr": "1",
                        "@Accession": "NC_000001.11",
                        "@start": "159204875",
                        "@stop": "159206500",
                        "@display_start": "159204875",
                        "@display_stop": "159206500",
                        "@Strand": "+",
                    },
                ],
                "XRef": [
                    {"@ID": "2532", "@DB": "Gene"},
                ],
            },
            "XRef": [
                {"@Type": "Allelic variant", "@ID": "613665.0004", "@DB": "OMIM"},
            ],
        }
    ],
)
def test_measure_from_json_data(value, snapshot):
    obj = models.Measure.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    # NB: the input/expect output value pairs do not fit on one screen any more for
    # this test, so we use snapshot testing
    snapshot.assert_match(json.dumps(value, indent=2), "output")


@pytest.mark.parametrize(
    "value",
    [
        {
            "@Type": "Variant",
            "@ID": "18397",
            "@Acc": "VCV000018397",
            "@Version": "1",
            "Measure": {
                "@Type": "Deletion",
                "@ID": "33436",
                "Name": [
                    {
                        "ElementValue": {
                            "@Type": "Preferred",
                            "#text": "NM_002036.4(ACKR1):c.286_299del (p.Trp96fs)",
                        }
                    },
                    {
                        "ElementValue": {"@Type": "Alternate", "#text": "ACKR1, 14-BP DEL, NT286"},
                        "XRef": {"@Type": "Allelic variant", "@ID": "613665.0004", "@DB": "OMIM"},
                    },
                ],
                "CanonicalSPDI": "NC_000001.11:159205718:CCTGGCTGGCCTGTCCTGGC:CCTGGC",
                "AttributeSet": [
                    {
                        "Attribute": {
                            "@Accession": "LRG_801t1",
                            "@Type": "HGVS, coding, LRG",
                            "#text": "LRG_801t1:c.286_299del",
                        }
                    },
                ],
                "CytogeneticLocation": "1q23.2",
                "SequenceLocation": [
                    {
                        "@Assembly": "GRCh38",
                        "@AssemblyAccessionVersion": "GCF_000001405.38",
                        "@AssemblyStatus": "current",
                        "@Chr": "1",
                        "@Accession": "NC_000001.11",
                        "@start": "159205719",
                        "@stop": "159205732",
                        "@display_start": "159205719",
                        "@display_stop": "159205732",
                        "@variantLength": "14",
                        "@positionVCF": "159205718",
                        "@referenceAlleleVCF": "CCCTGGCTGGCCTGT",
                        "@alternateAlleleVCF": "C",
                    },
                ],
                "MeasureRelationship": {
                    "@Type": "within single gene",
                    "Name": {
                        "ElementValue": {
                            "@Type": "Preferred",
                            "#text": "atypical chemokine receptor 1 (Duffy blood group)",
                        }
                    },
                    "Symbol": {"ElementValue": {"@Type": "Preferred", "#text": "ACKR1"}},
                    "SequenceLocation": [
                        {
                            "@Assembly": "GRCh38",
                            "@AssemblyAccessionVersion": "GCF_000001405.38",
                            "@AssemblyStatus": "current",
                            "@Chr": "1",
                            "@Accession": "NC_000001.11",
                            "@start": "159204875",
                            "@stop": "159206500",
                            "@display_start": "159204875",
                            "@display_stop": "159206500",
                            "@Strand": "+",
                        },
                    ],
                    "XRef": [
                        {"@ID": "2532", "@DB": "Gene"},
                    ],
                },
                "XRef": [
                    {"@Type": "Allelic variant", "@ID": "613665.0004", "@DB": "OMIM"},
                ],
            },
            "Name": {
                "ElementValue": {
                    "@Type": "Preferred",
                    "#text": "NM_002036.4(ACKR1):c.286_299del (p.Trp96fs)",
                }
            },
            "XRef": {"@ID": "CA113791", "@DB": "ClinGen"},
        },
    ],
)
def test_measure_set_from_json_data(value, snapshot):
    obj = models.MeasureSet.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    snapshot.assert_match(json.dumps(value, indent=2), "output")


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Type": "CompoundHeterozygote",
                "MeasureSet": [
                    {
                        "@Type": "Variant",
                        "Measure": {
                            "@Type": "Variation",
                            "AttributeSet": {
                                "Attribute": {"@Type": "HGVS", "#text": "NM_000497.3:c.1024C>T"}
                            },
                            "MeasureRelationship": {
                                "@Type": "variant in gene",
                                "Symbol": {
                                    "ElementValue": {"@Type": "Preferred", "#text": "CYP11B1"}
                                },
                            },
                        },
                    },
                    {
                        "@Type": "Variant",
                        "Measure": {
                            "@Type": "Variation",
                            "AttributeSet": {
                                "Attribute": {"@Type": "HGVS", "#text": "NM_000497.3:c.1012dup"}
                            },
                            "MeasureRelationship": {
                                "@Type": "variant in gene",
                                "Symbol": {
                                    "ElementValue": {"@Type": "Preferred", "#text": "CYP11B1"}
                                },
                            },
                        },
                    },
                ],
                "Name": {
                    "ElementValue": {
                        "@Type": "Preferred",
                        "#text": "NM_000497.3:c.[1024C>T];[1012dup]",
                    }
                },
            },
            {
                "measures": [
                    {
                        "measures": [
                            {
                                "attributes": [{"type": "HGVS", "value": "NM_000497.3:c.1024C>T"}],
                                "measure_relationship": [
                                    {
                                        "symbols": [
                                            {"value": {"type": "Preferred", "value": "CYP11B1"}}
                                        ]
                                    }
                                ],
                            }
                        ],
                        "type": "Variant",
                    },
                    {
                        "measures": [
                            {
                                "attributes": [{"type": "HGVS", "value": "NM_000497.3:c.1012dup"}],
                                "measure_relationship": [
                                    {
                                        "symbols": [
                                            {"value": {"type": "Preferred", "value": "CYP11B1"}}
                                        ]
                                    }
                                ],
                            }
                        ],
                        "type": "Variant",
                    },
                ],
                "names": [
                    {"value": {"type": "Preferred", "value": "NM_000497.3:c.[1024C>T];[1012dup]"}}
                ],
                "type": "CompoundHeterozygote",
            },
        )
    ],
)
def test_genotype_set_from_json_data(value, expected):
    obj = models.GenotypeSet.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@Type": "Disease",
                "Trait": {
                    "@Type": "Disease",
                    "Name": {
                        "ElementValue": {
                            "@Type": "Preferred",
                            "#text": "BARDET-BIEDL SYNDROME 2/6, DIGENIC",
                        }
                    },
                },
            },
            {
                "traits": [
                    {
                        "names": [
                            {
                                "value": {
                                    "type": "Preferred",
                                    "value": "BARDET-BIEDL SYNDROME 2/6, " "DIGENIC",
                                }
                            }
                        ],
                        "type": "Disease",
                    }
                ],
                "type": "Disease",
            },
        )
    ],
)
def test_trait_set_from_json_data(value, expected):
    obj = models.TraitSet.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected


@pytest.mark.parametrize(
    "value",
    [
        {
            "@ID": "63146",
            "@DateLastUpdated": "2022-04-23",
            "@DateCreated": "2013-04-04",
            "ClinVarAccession": {
                "@Acc": "RCV000005643",
                "@DateUpdated": "2022-04-23",
                "@DateCreated": "2013-04-04",
                "@Version": "4",
                "@Type": "RCV",
            },
            "RecordStatus": "current",
            "ClinicalSignificance": {
                "@DateLastEvaluated": "2001-09-21",
                "ReviewStatus": "no assertion criteria provided",
                "Description": "Pathogenic",
            },
            "Assertion": {"@Type": "variation to disease"},
            "ObservedIn": {
                "Sample": {
                    "Origin": "germline",
                    "Species": {"@TaxonomyId": "9606", "#text": "human"},
                    "AffectedStatus": "not provided",
                },
                "Method": {"MethodType": "literature only"},
                "ObservedData": {
                    "@ID": "96578166",
                    "Attribute": {
                        "@Type": "Description",
                        "#text": "In a patient carrying 2 different termination codons in the BBS2 gene (606151.0003, 606151.0004), Katsanis et al. (2001) identified a nonsense mutation in the BBS6 gene, a glutamine-to-termination substitution at codon 147.",
                    },
                    "Citation": {
                        "@Type": "general",
                        "ID": {"@Source": "PubMed", "#text": "11567139"},
                    },
                },
            },
            "MeasureSet": {
                "@Type": "Variant",
                "@ID": "5318",
                "@Acc": "VCV000005318",
                "@Version": "1",
                "Measure": {
                    "@Type": "single nucleotide variant",
                    "@ID": "20357",
                    "Name": [
                        {
                            "ElementValue": {
                                "@Type": "Preferred",
                                "#text": "NM_170784.3(MKKS):c.442C>T (p.Gln148Ter)",
                            }
                        },
                        {
                            "ElementValue": {"@Type": "Alternate", "#text": "Q147*"},
                            "XRef": {
                                "@Type": "Allelic variant",
                                "@ID": "604896.0012",
                                "@DB": "OMIM",
                            },
                        },
                    ],
                    "CanonicalSPDI": "NC_000020.11:10413072:G:A",
                    "AttributeSet": [
                        {
                            "Attribute": {
                                "@Accession": "NM_018848",
                                "@Version": "3",
                                "@Change": "c.442C>T",
                                "@Type": "HGVS, coding, RefSeq",
                                "#text": "NM_018848.3:c.442C>T",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NM_170784",
                                "@Version": "3",
                                "@Change": "c.442C>T",
                                "@Type": "HGVS, coding, RefSeq",
                                "@MANESelect": "true",
                                "#text": "NM_170784.3:c.442C>T",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NG_009109",
                                "@Version": "2",
                                "@Change": "g.26146C>T",
                                "@Type": "HGVS, genomic, RefSeqGene",
                                "#text": "NG_009109.2:g.26146C>T",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NC_000020",
                                "@Version": "11",
                                "@Change": "g.10413073G>A",
                                "@Type": "HGVS, genomic, top level",
                                "@integerValue": "38",
                                "#text": "NC_000020.11:g.10413073G>A",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NC_000020",
                                "@Version": "10",
                                "@Change": "g.10393721G>A",
                                "@Type": "HGVS, genomic, top level, previous",
                                "@integerValue": "37",
                                "#text": "NC_000020.10:g.10393721G>A",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NP_061336",
                                "@Version": "1",
                                "@Change": "p.Gln148Ter",
                                "@Type": "HGVS, protein, RefSeq",
                                "#text": "NP_061336.1:p.Gln148Ter",
                            }
                        },
                        {
                            "Attribute": {
                                "@Accession": "NP_740754",
                                "@Version": "1",
                                "@Change": "p.Gln148Ter",
                                "@Type": "HGVS, protein, RefSeq",
                                "#text": "NP_740754.1:p.Gln148Ter",
                            }
                        },
                        {
                            "Attribute": {"@Type": "MolecularConsequence", "#text": "nonsense"},
                            "XRef": [
                                {"@ID": "SO:0001587", "@DB": "Sequence Ontology"},
                                {"@ID": "NM_018848.3:c.442C>T", "@DB": "RefSeq"},
                            ],
                        },
                        {
                            "Attribute": {"@Type": "MolecularConsequence", "#text": "nonsense"},
                            "XRef": [
                                {"@ID": "SO:0001587", "@DB": "Sequence Ontology"},
                                {"@ID": "NM_170784.3:c.442C>T", "@DB": "RefSeq"},
                            ],
                        },
                        {"Attribute": {"@Type": "ProteinChange1LetterCode", "#text": "Q148*"}},
                        {"Attribute": {"@Type": "ProteinChange3LetterCode", "#text": "GLN147TER"}},
                    ],
                    "CytogeneticLocation": "20p12.2",
                    "SequenceLocation": [
                        {
                            "@Assembly": "GRCh38",
                            "@AssemblyAccessionVersion": "GCF_000001405.38",
                            "@AssemblyStatus": "current",
                            "@Chr": "20",
                            "@Accession": "NC_000020.11",
                            "@start": "10413073",
                            "@stop": "10413073",
                            "@display_start": "10413073",
                            "@display_stop": "10413073",
                            "@variantLength": "1",
                            "@positionVCF": "10413073",
                            "@referenceAlleleVCF": "G",
                            "@alternateAlleleVCF": "A",
                        },
                        {
                            "@Assembly": "GRCh37",
                            "@AssemblyAccessionVersion": "GCF_000001405.25",
                            "@AssemblyStatus": "previous",
                            "@Chr": "20",
                            "@Accession": "NC_000020.10",
                            "@start": "10393721",
                            "@stop": "10393721",
                            "@display_start": "10393721",
                            "@display_stop": "10393721",
                            "@variantLength": "1",
                            "@positionVCF": "10393721",
                            "@referenceAlleleVCF": "G",
                            "@alternateAlleleVCF": "A",
                        },
                    ],
                    "MeasureRelationship": {
                        "@Type": "within single gene",
                        "Name": {
                            "ElementValue": {
                                "@Type": "Preferred",
                                "#text": "MKKS centrosomal shuttling protein",
                            }
                        },
                        "Symbol": {"ElementValue": {"@Type": "Preferred", "#text": "MKKS"}},
                        "SequenceLocation": [
                            {
                                "@Assembly": "GRCh38",
                                "@AssemblyAccessionVersion": "GCF_000001405.38",
                                "@AssemblyStatus": "current",
                                "@Chr": "20",
                                "@Accession": "NC_000020.11",
                                "@start": "10401009",
                                "@stop": "10434222",
                                "@display_start": "10401009",
                                "@display_stop": "10434222",
                                "@Strand": "-",
                            },
                            {
                                "@Assembly": "GRCh37",
                                "@AssemblyAccessionVersion": "GCF_000001405.25",
                                "@AssemblyStatus": "previous",
                                "@Chr": "20",
                                "@Accession": "NC_000020.10",
                                "@start": "10385427",
                                "@stop": "10414886",
                                "@display_start": "10385427",
                                "@display_stop": "10414886",
                                "@variantLength": "29460",
                                "@Strand": "-",
                            },
                        ],
                        "XRef": [
                            {"@ID": "8195", "@DB": "Gene"},
                            {"@Type": "MIM", "@ID": "604896", "@DB": "OMIM"},
                            {"@ID": "HGNC:7108", "@DB": "HGNC"},
                        ],
                    },
                    "XRef": [
                        {"@Type": "Allelic variant", "@ID": "604896.0012", "@DB": "OMIM"},
                        {"@Type": "rs", "@ID": "137853154", "@DB": "dbSNP"},
                    ],
                },
                "Name": {
                    "ElementValue": {
                        "@Type": "Preferred",
                        "#text": "NM_170784.3(MKKS):c.442C>T (p.Gln148Ter)",
                    }
                },
                "XRef": {"@ID": "CA117400", "@DB": "ClinGen"},
            },
            "TraitSet": {
                "@Type": "Disease",
                "@ID": "19803",
                "Trait": {
                    "@ID": "32093",
                    "@Type": "Disease",
                    "Name": {
                        "ElementValue": {
                            "@Type": "Preferred",
                            "#text": "Bardet-biedl syndrome 2/6, digenic",
                        }
                    },
                    "XRef": {"@ID": "C4016908", "@DB": "MedGen"},
                },
            },
        }
    ],
)
def test_reference_clinvar_assertion_from_json_data(value, snapshot):
    obj = models.ReferenceClinVarAssertion.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    snapshot.assert_match(json.dumps(value, indent=2, default=conversion.json_default), "output")


@pytest.mark.parametrize(
    "value,expected",
    [
        (
            {
                "@localKey": "604896.0012_BARDET-BIEDL SYNDROME 2/6, DIGENIC",
                "@submitter": "OMIM",
                "@submitterDate": "2017-12-13",
                "@title": "MKKS, GLN147TER_BARDET-BIEDL SYNDROME 2/6, DIGENIC",
            },
            {
                "local_key": "604896.0012_BARDET-BIEDL SYNDROME 2/6, DIGENIC",
                "submitter": "OMIM",
                "submitter_date": datetime.date(2017, 12, 13),
                "title": "MKKS, GLN147TER_BARDET-BIEDL SYNDROME 2/6, DIGENIC",
            },
        ),
    ],
)
def test_clinvar_submission_id_from_json_data(value, expected):
    obj = models.ClinVarSubmissionId.from_json_data(value)
    value = conversion.remove_empties_from_containers(cattrs.unstructure(obj))
    assert value == expected



import pytest

from clinvar_api import exceptions


def test_clinvar_api_exception():
    with pytest.raises(exceptions.ClinvarApiException):
        raise exceptions.ClinvarApiException()


def test_submission_failed():
    with pytest.raises(exceptions.SubmissionFailed):
        raise exceptions.SubmissionFailed()


def test_query_failed():
    with pytest.raises(exceptions.QueryFailed):
        raise exceptions.QueryFailed()



import pathlib

import pytest

from clinvar_api.msg import (
    Assembly,
    Chromosome,
    ClinicalSignificanceDescription,
    ModeOfInheritance,
    VariantType,
)
from clinvar_this import exceptions
from clinvar_this.io.tsv import (
    SeqVarTsvRecord,
    StrucVarTsvRecord,
    TsvType,
    guess_tsv_type,
    read_seq_var_tsv,
    read_struc_var_tsv,
)

DATA_DIR = pathlib.Path(__file__).parent / "data/io_tsv"


def test_read_seq_var_tsv_path():
    actual = read_seq_var_tsv(path=DATA_DIR / "example.tsv")
    assert actual == [
        SeqVarTsvRecord(
            assembly=Assembly.GRCH37,
            chromosome=Chromosome.CHR10,
            pos=115614632,
            ref="A",
            alt="G",
            omim=["OMIM:618278"],
            inheritance=ModeOfInheritance.AUTOSOMAL_RECESSIVE_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            local_key="KEY",
            extra_data={"gene": "NHLRC2"},
        )
    ]


def test_read_struc_var_tsv_path():
    actual = read_struc_var_tsv(path=DATA_DIR / "example_sv.tsv")
    assert actual == [
        StrucVarTsvRecord(
            assembly=Assembly.GRCH38,
            chromosome=Chromosome.CHR1,
            start=844347,
            stop=4398122,
            sv_type=VariantType.DELETION,
            omim=[],
            inheritance=ModeOfInheritance.AUTOSOMAL_DOMINANT_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            hpo_terms=["HP:0001263"],
        )
    ]


def test_read_seq_var_tsv_file():
    with (DATA_DIR / "example.tsv").open("rt") as inputf:
        actual = read_seq_var_tsv(file=inputf)
    assert actual == [
        SeqVarTsvRecord(
            assembly=Assembly.GRCH37,
            chromosome=Chromosome.CHR10,
            pos=115614632,
            ref="A",
            alt="G",
            omim=["OMIM:618278"],
            inheritance=ModeOfInheritance.AUTOSOMAL_RECESSIVE_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            local_key="KEY",
            extra_data={"gene": "NHLRC2"},
        )
    ]


def test_read_struc_var_tsv_file():
    with (DATA_DIR / "example_sv.tsv").open("rt") as inputf:
        actual = read_struc_var_tsv(file=inputf)
    assert actual == [
        StrucVarTsvRecord(
            assembly=Assembly.GRCH38,
            chromosome=Chromosome.CHR1,
            start=844347,
            stop=4398122,
            sv_type=VariantType.DELETION,
            omim=[],
            inheritance=ModeOfInheritance.AUTOSOMAL_DOMINANT_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            hpo_terms=["HP:0001263"],
        )
    ]


def test_read_seq_var_tsv_path_bad():
    with pytest.raises(exceptions.InvalidFormat):
        read_seq_var_tsv(path=DATA_DIR / "example.bad.tsv")


def test_read_struc_var_tsv_path_bad():
    with pytest.raises(exceptions.InvalidFormat):
        read_struc_var_tsv(path=DATA_DIR / "example_sv.bad.tsv")


def test_read_seq_var_tsv_error():
    with pytest.raises(TypeError):
        read_seq_var_tsv()


def test_read_struc_var_tsv_error():
    with pytest.raises(TypeError):
        read_struc_var_tsv()


def test_guess_file_type():
    assert guess_tsv_type(path=DATA_DIR / "example.bad.tsv") == None  # noqa: E711
    assert guess_tsv_type(path=DATA_DIR / "example.tsv") == TsvType.SEQ_VAR
    assert guess_tsv_type(path=DATA_DIR / "example_sv.tsv") == TsvType.STRUC_VAR



"""Test ``clinar_api.msg`` with real-world examples."""

from clinvar_api import common, msg


def test_structure_data_created(data_created):
    assert common.CONVERTER.structure(data_created, msg.Created)


def test_structure_data_message(data_message):
    assert common.CONVERTER.structure(data_message, msg.Error)


def test_data_submission_submitted(data_submission_submitted):
    assert common.CONVERTER.structure(data_submission_submitted, msg.SubmissionStatus)


def test_data_submission_processing(data_submission_processing):
    assert common.CONVERTER.structure(data_submission_processing, msg.SubmissionStatus)


def test_data_submission_processed(data_submission_processed):
    assert common.CONVERTER.structure(data_submission_processed, msg.SubmissionStatus)


def test_data_partially_successful_submission(data_partially_successful_submission):
    assert common.CONVERTER.structure(data_partially_successful_submission, msg.SubmissionStatus)


def test_data_summary_response_processed(data_summary_response_processed):
    assert common.CONVERTER.structure(data_summary_response_processed, msg.SummaryResponse)


def test_data_summary_response_error_partial(data_summary_response_error_partial):
    assert common.CONVERTER.structure(data_summary_response_error_partial, msg.SummaryResponse)


def test_data_summary_response_error_all(data_summary_response_error_all):
    assert common.CONVERTER.structure(data_summary_response_error_all, msg.SummaryResponse)


def test_data_submission_snv(data_submission_snv):
    assert common.CONVERTER.structure(data_submission_snv, msg.SubmissionContainer)



from clinvar_data import phenotype_link


def test_smoke_test_run(tmpdir, snapshot):
    path_input = "tests/clinvar_data/data/records_with_hpo.jsonl"
    path_output = f"{tmpdir}/output.jsonl"
    phenotype_link.run_report(path_input, path_output)

    with open(path_output, "rt") as outputf:
        snapshot.assert_match(outputf.read(), "output.jsonl")



import json
import os
import pathlib
from unittest.mock import MagicMock

from freezegun import freeze_time
import pytest

import clinvar_api
from clinvar_api import models
from clinvar_api.client import RetrieveStatusResult
from clinvar_api.common import CONVERTER
from clinvar_this import batches, exceptions
from clinvar_this.io import tsv as io_tsv

# We must read this outside of the test as we use the fake file system.
with (pathlib.Path(__file__).parent / "data/batches/seq_variant.tsv").open("rt") as inputf:
    #: Small variant TSV for testing batches module.  Likely pathogenic.
    SEQ_VARIANT_TSV = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/struc_variant.tsv").open("rt") as inputf:
    #: Structural variant TSV for testing batches module.  Likely pathogenic.
    STRUC_VARIANT_TSV = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/seq_variant.payload.json").open("rt") as inputf:
    #: The ``SEQ_VARIANT_TSV`` after import for testing batches module.
    SEQ_VARIANT_PAYLOAD_JSON = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/struc_variant.payload.json").open(
    "rt"
) as inputf:
    #: The ``STRUC_VARIANT_TSV`` after import for testing batches module.
    STRUC_VARIANT_PAYLOAD_JSON = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/seq_variant-update.tsv").open("rt") as inputf:
    #: Sequence variant TSV for testing batches module.  Updated to Pathonenic.
    SEQ_VARIANT_UPDATE_TSV = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/struc_variant-update.tsv").open("rt") as inputf:
    #: Structural variant TSV for testing batches module.  Updated to Pathonenic.
    STRUC_VARIANT_UPDATE_TSV = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/seq_variant-update.payload.json").open(
    "rt"
) as inputf:
    #: The `SEQ_VARIANT_UPDATE_TSV` after import / merge.
    SEQ_VARIANT_UPDATE_PAYLOAD_JSON = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/struc_variant-update.payload.json").open(
    "rt"
) as inputf:
    #: The `STRUC_VARIANT_UPDATE_TSV` after import / merge.
    STRUC_VARIANT_UPDATE_PAYLOAD_JSON = inputf.read()

with (
    pathlib.Path(__file__).parent / "data/batches/seq_variant.retrieve-response-processing.json"
).open("rt") as inputf:
    #: A static response on retrieving with "processing" status.
    SEQ_VARIANT_RETRIEVE_RESPONSE_PROCESSING_JSON = inputf.read()

with (
    pathlib.Path(__file__).parent / "data/batches/seq_variant.retrieve-response-submitted.json"
).open("rt") as inputf:
    #: A static response on retrieving with "submitted" status.
    SEQ_VARIANT_RETRIEVE_RESPONSE_SUBMITTED_JSON = inputf.read()

with (
    pathlib.Path(__file__).parent / "data/batches/seq_variant.retrieve-response-success.json"
).open("rt") as inputf:
    #: A static response on retrieving with "success" status.
    SEQ_VARIANT_RETRIEVE_RESPONSE_SUCCESS_JSON = inputf.read()

with (pathlib.Path(__file__).parent / "data/batches/seq_variant.retrieve-response-error.json").open(
    "rt"
) as inputf:
    #: A static response on retrieving with "error" status.
    SEQ_VARIANT_RETRIEVE_RESPONSE_ERROR_JSON = inputf.read()

SUBMISSION_SCHEMA_JSON_PATH = (
    pathlib.Path(clinvar_api.__file__).parent / "schemas/submission_schema.json"
)
with SUBMISSION_SCHEMA_JSON_PATH.open("rt") as inputf:
    SUBMISSION_SCHEMA_JSON = inputf.read()


def test_list_no_batches_no_dir(fs, app_config, capsys):
    batches.list_(app_config)

    captured = capsys.readouterr()
    assert "-- NO BATCHES YET --" in captured.out


def test_list_no_batches_empty_repository(fs, app_config, capsys):
    fs.create_dir(
        os.path.expanduser("~/.local/share/clinvar-this/default"),
    )

    batches.list_(app_config)

    captured = capsys.readouterr()
    assert "-- NO BATCHES YET --" in captured.out


def test_list_with_batches(fs, app_config, capsys):
    fs.create_dir(
        os.path.expanduser("~/.local/share/clinvar-this/default/one"),
    )
    fs.create_dir(
        os.path.expanduser("~/.local/share/clinvar-this/default/two"),
    )

    batches.list_(app_config)

    captured = capsys.readouterr()
    assert "one" in captured.out
    assert "two" in captured.out


@freeze_time("2012-01-14")
def test_gen_name(fs, app_config):
    name = batches.gen_name(app_config)
    assert name == "2012-01-14-000"
    fs.create_dir(
        os.path.expanduser("~/.local/share/clinvar-this/default/2012-01-14-000"),
    )
    name = batches.gen_name(app_config)
    assert name == "2012-01-14-001"


def test_merge_submission_container():
    pass  # TODO:  more comprehensive tests


@freeze_time("2012-01-14")
def test_import_seq_variant_tsv_new(fs, app_config, monkeypatch):
    path_tsv = "/tmp/seq_variant.tsv"
    fs.create_file(path_tsv, contents=SEQ_VARIANT_TSV)

    def mock_uuid4():
        return "mock-uuid4"

    monkeypatch.setattr(io_tsv.uuid, "uuid4", mock_uuid4)

    batch_name = "the-batch"
    batches.import_(config=app_config, name=batch_name, path=path_tsv, metadata=())

    payload_path = os.path.expanduser(
        "~/.local/share/clinvar-this/default/the-batch/payload.20120114000000.json"
    )
    assert os.path.exists(payload_path)
    with open(payload_path, "rt") as inputf:
        payload_json = inputf.read()
    assert payload_json == SEQ_VARIANT_PAYLOAD_JSON


@freeze_time("2012-01-14")
def test_import_seq_variant_tsv_update(fs, app_config):
    path_tsv = "/tmp/seq_variant-update.tsv"
    fs.create_file(path_tsv, contents=SEQ_VARIANT_UPDATE_TSV)

    payload_path = os.path.expanduser(
        "~/.local/share/clinvar-this/default/the-batch/payload.20120114000000.json"
    )
    fs.create_file(
        os.path.expanduser(payload_path),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )

    batch_name = "the-batch"
    batches.import_(config=app_config, name=batch_name, path=path_tsv, metadata=())

    filename = os.listdir(os.path.expanduser("~/.local/share/clinvar-this/default/the-batch"))[-1]

    payload_path = os.path.expanduser(f"~/.local/share/clinvar-this/default/the-batch/{filename}")
    assert os.path.exists(payload_path)
    with open(payload_path, "rt") as inputf:
        payload_json = inputf.read()
    assert payload_json == SEQ_VARIANT_UPDATE_PAYLOAD_JSON


@freeze_time("2012-01-14")
def test_import_deletion_tsv_new(fs):
    pass  # TODO


@freeze_time("2012-01-14")
def test_import_deletion_tsv_update(fs):
    pass  # TODO


@freeze_time("2012-01-14")
def test_import_structural_variant_tsv_new(fs, app_config, monkeypatch):
    path_tsv = "/tmp/seq_variant.tsv"
    fs.create_file(path_tsv, contents=STRUC_VARIANT_TSV)

    def mock_uuid4():
        return "mock-uuid4"

    monkeypatch.setattr(io_tsv.uuid, "uuid4", mock_uuid4)

    batch_name = "the-batch"
    batches.import_(config=app_config, name=batch_name, path=path_tsv, metadata=())

    payload_path = os.path.expanduser(
        "~/.local/share/clinvar-this/default/the-batch/payload.20120114000000.json"
    )
    assert os.path.exists(payload_path)
    with open(payload_path, "rt") as inputf:
        payload_json = inputf.read()
    assert payload_json == STRUC_VARIANT_PAYLOAD_JSON


def test_import_structural_variant_tsv_update(fs, app_config):
    path_tsv = "/tmp/struc_variant-update.tsv"
    fs.create_file(path_tsv, contents=STRUC_VARIANT_UPDATE_TSV)

    payload_path = os.path.expanduser(
        "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
    )
    fs.create_file(
        os.path.expanduser(payload_path),
        contents=STRUC_VARIANT_PAYLOAD_JSON,
    )
    assert os.path.exists(payload_path)

    batch_name = "the-batch"
    batches.import_(config=app_config, name=batch_name, path=path_tsv, metadata=())

    fname = os.listdir(os.path.expanduser("~/.local/share/clinvar-this/default/the-batch"))[-1]

    payload_path = os.path.expanduser(f"~/.local/share/clinvar-this/default/the-batch/{fname}")
    assert os.path.exists(payload_path)
    with open(payload_path, "rt") as inputf:
        payload_json = inputf.read()
    assert payload_json == STRUC_VARIANT_UPDATE_PAYLOAD_JSON


@pytest.mark.parametrize(
    "exists,force",
    [
        (False, False),
        (True, False),
        (True, True),
    ],
)
def test_export_seq_variant_tsv(fs, app_config, exists, force):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )

    if exists:
        fs.create_file("/tmp/output.tsv", contents="foo")

    batch_name = "the-batch"

    if exists and not force:
        with pytest.raises(exceptions.IOException):
            batches.export(config=app_config, name=batch_name, path="/tmp/output.tsv")
    else:
        batches.export(config=app_config, name=batch_name, path="/tmp/output.tsv", force=force)

    with open("/tmp/output.tsv", "rt") as inputf:
        fcontents = inputf.read()

    if not exists or force:
        expected = "\n".join(
            [
                "ASSEMBLY\tCHROM\tPOS\tREF\tALT\tOMIM\tMOI\tCLIN_SIG\tCLIN_EVAL\tCLIN_COMMENT\tKEY\tHPO",
                (
                    "GRCh37\t19\t48183936\tC\tCA\t619325\tAutosomal dominant inheritance\t"
                    "Likely pathogenic\t\t\t\tHP:0004322,HP:0001263\n"
                ),
            ]
        )
    else:
        expected = "foo"
    assert fcontents == expected


@pytest.mark.parametrize(
    "exists,force",
    [
        (False, False),
        (True, False),
        (True, True),
    ],
)
def test_export_structural_variant_tsv(fs, app_config, exists, force):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=STRUC_VARIANT_PAYLOAD_JSON,
    )

    if exists:
        fs.create_file("/tmp/output.tsv", contents="foo")

    batch_name = "the-batch"

    if exists and not force:
        with pytest.raises(exceptions.IOException):
            batches.export(
                config=app_config, name=batch_name, path="/tmp/output.tsv", struc_var=True
            )
    else:
        batches.export(
            config=app_config, name=batch_name, path="/tmp/output.tsv", force=force, struc_var=True
        )

    with open("/tmp/output.tsv", "rt") as inputf:
        fcontents = inputf.read()

    if not exists or force:
        expected = "\n".join(
            [
                (
                    "ASSEMBLY\tCHROM\tSTART\tSTOP\tSV_TYPE\tOMIM\tMOI\tCLIN_SIG\tCLIN_EVAL\t"
                    "CLIN_COMMENT\tKEY\tHPO"
                ),
                (
                    "GRCh38\t1\t844347\t4398122\tDeletion\t\tAutosomal dominant inheritance\t"
                    "not provided\t\t\t\t\n"
                ),
            ]
        )
    else:
        expected = "foo"
    assert fcontents == expected


@pytest.mark.parametrize(
    "use_testing,dry_run",
    [(False, False), (False, True)],
)
@freeze_time("2012-01-14")
def test_submit(fs, app_config, use_testing, dry_run, monkeypatch):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )

    fs.create_file(SUBMISSION_SCHEMA_JSON_PATH, contents=SUBMISSION_SCHEMA_JSON)

    def mock_submit_data(_self, _payload):
        return {"id": "SUB000fake"}

    monkeypatch.setattr(batches.client.Client, "submit_data", mock_submit_data)

    batches.submit(config=app_config, name="the-batch", use_testing=use_testing, dry_run=dry_run)

    response_path = os.path.expanduser(
        "~/.local/share/clinvar-this/default/the-batch/submission-response.20120114000000.json"
    )
    if not dry_run:
        assert os.path.exists(response_path)
        with open(response_path, "rt") as inputf:
            assert inputf.read() == '{"id": "SUB000fake"}'
    else:
        assert not os.path.exists(response_path)


def test_retrieve_state_submitted(fs, app_config, monkeypatch):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/submission-response.20120114000000.json"
        ),
        contents='{"id": "SUB000fake"}',
    )

    response = json.loads(SEQ_VARIANT_RETRIEVE_RESPONSE_SUBMITTED_JSON)

    mock_retrieve_status = MagicMock()
    mock_retrieve_status.return_value = RetrieveStatusResult(
        status=CONVERTER.structure(response["status"], models.SubmissionStatus), summaries={}
    )

    monkeypatch.setattr(batches.client.Client, "retrieve_status", mock_retrieve_status)

    batches.retrieve(config=app_config, name="the-batch", use_testing=False)

    mock_retrieve_status.assert_called_once()
    assert len(mock_retrieve_status.call_args.args) == 1
    assert len(mock_retrieve_status.call_args.kwargs) == 0
    assert mock_retrieve_status.call_args.args[0] == "SUB000fake"


def test_retrieve_state_processing(fs, app_config, monkeypatch):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/submission-response.20120114000000.json"
        ),
        contents='{"id": "SUB000fake"}',
    )

    response = json.loads(SEQ_VARIANT_RETRIEVE_RESPONSE_PROCESSING_JSON)

    mock_retrieve_status = MagicMock()
    mock_retrieve_status.return_value = RetrieveStatusResult(
        status=CONVERTER.structure(response["status"], models.SubmissionStatus), summaries={}
    )

    monkeypatch.setattr(batches.client.Client, "retrieve_status", mock_retrieve_status)

    batches.retrieve(config=app_config, name="the-batch", use_testing=False)

    mock_retrieve_status.assert_called_once()
    assert len(mock_retrieve_status.call_args.args) == 1
    assert len(mock_retrieve_status.call_args.kwargs) == 0
    assert mock_retrieve_status.call_args.args[0] == "SUB000fake"


def test_retrieve_state_processed(fs, app_config, monkeypatch):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/submission-response.20120114000000.json"
        ),
        contents='{"id": "SUB000fake"}',
    )

    response = json.loads(SEQ_VARIANT_RETRIEVE_RESPONSE_SUCCESS_JSON)

    mock_retrieve_status = MagicMock()
    mock_retrieve_status.return_value = RetrieveStatusResult(
        status=CONVERTER.structure(response["status"], models.SubmissionStatus),
        summaries={
            key: CONVERTER.structure(value, models.SummaryResponse)
            for key, value in response["summaries"].items()
        },
    )

    monkeypatch.setattr(batches.client.Client, "retrieve_status", mock_retrieve_status)

    batches.retrieve(config=app_config, name="the-batch", use_testing=False)

    mock_retrieve_status.assert_called_once()
    assert len(mock_retrieve_status.call_args.args) == 1
    assert len(mock_retrieve_status.call_args.kwargs) == 0
    assert mock_retrieve_status.call_args.args[0] == "SUB000fake"


def test_retrieve_state_error(fs, app_config, monkeypatch):
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/payload.20120113000000.json"
        ),
        contents=SEQ_VARIANT_PAYLOAD_JSON,
    )
    fs.create_file(
        os.path.expanduser(
            "~/.local/share/clinvar-this/default/the-batch/submission-response.20120114000000.json"
        ),
        contents='{"id": "SUB000fake"}',
    )

    response = json.loads(SEQ_VARIANT_RETRIEVE_RESPONSE_ERROR_JSON)

    mock_retrieve_status = MagicMock()
    mock_retrieve_status.return_value = RetrieveStatusResult(
        status=CONVERTER.structure(response["status"], models.SubmissionStatus), summaries={}
    )

    monkeypatch.setattr(batches.client.Client, "retrieve_status", mock_retrieve_status)

    batches.retrieve(config=app_config, name="the-batch", use_testing=False)

    mock_retrieve_status.assert_called_once()
    assert len(mock_retrieve_status.call_args.args) == 1
    assert len(mock_retrieve_status.call_args.kwargs) == 0
    assert mock_retrieve_status.call_args.args[0] == "SUB000fake"



"""Tests for the conversion module."""

import os

import pytest

from clinvar_data import conversion


@pytest.mark.parametrize(
    "value,expected",
    [
        ({}, None),
        ({"a": 1, "b": 2}, {"a": 1, "b": 2}),
        ({"a": None}, None),
        ({"a": {"b": None}}, None),
    ],
)
def test_remove_empties_from_dict(value, expected):
    assert conversion.remove_empties_from_containers(value) == expected


def test_cli_xml_to_json(tmp_path, snapshot):
    in_path = os.path.dirname(__file__) + "/data/one_record.xml"
    out_path = f"{tmp_path}/one_record.jsonl"
    conversion.convert(
        in_path,
        out_path,
        use_click=False,
    )

    with open(out_path, "rt") as output_f:
        snapshot.assert_match(output_f.read(), "one_record.jsonl")
